function [destination,null_event,time_limit,out_of_bound,terminal] = ...
    compute_mapping_DHA_VR(X0,sttype,Tstamp,srcloc,srccell)

% Compute the `mapping set` from the given polytope in the specified location.
%
% Syntax:
%   "[destination,null_event,time_limit,out_of_bound,terminal] = ...
%			compute_mapping_DHA(X0,sttype,Tstamp,srcloc,srccell,param_file)"
%
% Description:
%
%   The routine compute_mapping_DHA_VR performs the reachability from a initial set X0, finding  the 
%   set of states that can be reached, using variable-rate sampled-data analysis (i.e. discrete-time, difference
%   equations with sample periods that are a function of the continuous-valued state variables).
%   It also finds a conservative approximation of the elapsed time to reach them. 
%
%	 After this, it applies the reset function defined for this transition. As a last
%   operation, it wraps up the set of states with a conservative bounding box, avoiding explosion 
%   in the number of vertices and faces. 
%
%   The inputs to this function are: 
%
%   * "X0": a "linearcon" object representing the initial continuous set
%     for which the mapping is to be computed.
%  
%   * "sttype": type of state for "X0" which is either an `initial` or
%     `face` state (apply the initial displacement or not).
%
%	 * "Tstamp": vector with the time uncertainty to reach the region
% 
%   * "srcloc": source location.
% 
%	 * "srccell": cell region of source
%
%   The outputs of this function are
%
%   % `destination is a cell array with the following fields:
%
%   * dstloc 	: location whose guard was violated during the reachability process
%	 * dstcell 	: Interior region of the dstloc
%   * region 	: linearcon object representing the set of states that violates some guard for a given
%     location.
%   * Tstamp 	: time elapsed interval to reach the region starting at the set X0.
%
%   * "null_event": a boolean flag indicating that the flow pipe computation
%     was terminated because it can be concluded that the subsequent flow
%     pipe segments will remain inside "INV" forever.
%
%   * "time_limit": a boolean flag indicating that the flow pipe computation
%     was terminated because the time limit "max_time" was exceeded.
%
% Implementation:
%   Call the mapping computation routine corresponding to the type of the
%   composite continuous dynamics of all SCSBs for the given location
%   ("fs_lin_map" for `linear` (affine) dynamics, "fs_nonlin_map" for
%   `nonlinear` dynamics, and "clk_map" for `clock` dynamics). %   
%
% See Also:
%   fs_nonlin_map,fs_lin_map,clk_map

global GLOBAL_PIHA
global GLOBAL_APPROX_PARAM
PLOT=1;
% Get the invariant polytope for the given cell location.
INV = return_invariant(srccell);

% Compute the type of the composite dynamics of all SCSBs for the FSM
% state in the source location.

overall_dynamics = check_overall_dynamics(GLOBAL_PIHA.SCSBlocks, ...
    GLOBAL_PIHA.Locations{srcloc}.q);
 
max_time = GLOBAL_APPROX_PARAM.max_time;

null_event = 0;
time_limit = 0;
terminal = {};
out_of_bound = 0

equilibrium=0;
check_equilibrium=0;
check_time_limit=0;

% Get information about analysis region
for k=1:GLOBAL_PIHA.NAR
   c_AR(k,:) = GLOBAL_PIHA.Hyperplanes{k}.c;
   d_AR(k,:) = GLOBAL_PIHA.Hyperplanes{k}.d;
end
AR = linearcon([],[],c_AR,d_AR);


%For now, we treat all systems as nonlinear.
overall_dynamics = 'nonlinear'

switch overall_dynamics,
   
case 'linear',
   
        
case 'clock',
    v = overall_system_clock(GLOBAL_PIHA.SCSBlocks, ...
        GLOBAL_PIHA.Locations{srcloc}.q);
    
    
 case 'nonlinear',
    ode_param = GLOBAL_PIHA.Locations{srcloc}.q;
    check_time_limit = 1;
    
    
 otherwise,
    error(['Unknown dynamics type ''' overall_dynamics '''.'])
    
 end



fprintf(1,'Computing flow pipe segments:')
counter = 0;

global_var;
%PLOT = 0;

switch sttype
   
	case 'init', % initial state
      initial = 1;
      
	case 'face', % face state (a patch)
      initial=0;
          
   otherwise,
      error(['Unknown state type ''' sttype '''.'])
      return
end



%equilibrium = 0; time_limit = 0;Xsim = vertices;
%first = 1; t_total = 0; stop = 0;init_T=T;

% Compute the transformation of the initial segment

time_pointer = 0;
mapping_counter =1;
dst_struc = {};

% Currently, CheckMate only supports one variable-rate clock structure.
for i=1:1
   Pk_reach = X0;
   END_SEARCH = 0;       
	while ~END_SEARCH
      transition_found=0;
      
      % ----- Advance one step of clock  ---------% 
      
      
      if ~isempty(Pk_reach)
         switch overall_dynamics
         
	      	case 'linear'
                
                
            
		    case 'clock'
	   	      
	      
            case 'nonlinear'
               
   	         sample_points_X0 = vertices(Pk_reach);
      	      Pk = linearcon(Pk_reach);          
         	   [Pk,sample_points] = seg_approx_VRClock('overall_system_ode',ode_param,...
            	      Pk,sample_points_X0);
					Pk_reach = Pk;                 
               
          end 	%switch%
          if PLOT        
            drawnow;plot(polyhedron(Pk_reach));  
          end
       
      end	%if%
      
      if ~isempty(Pk_reach)
         
         % -- test if Pk_reach left cell -- %
      	if ~isempty(minus(Pk_reach,return_invariant(srccell)))
               
         	% -- Cycle through possible transitions -- %             
            for m=1:length(GLOBAL_PIHA.Locations{srcloc}.transitions)
                
          		% -- Cycle through each guard cell for each possible transition -- %             
                for k=1:length(GLOBAL_PIHA.Locations{srcloc}.transitions{m}.guard)
                   
                  % -- Test if guard cell was reached by Pk_reach -- %             
						if ~isempty(and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.transitions{m}.guard(k))))
                     
                     % -- compute intersection between Pk_reach and the guard cell -- %             
                     mapping  = and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations...
                        {srcloc}.transitions{m}.guard(k)));
                     
                     % -- Test if transitions refers to clock used in this loop -- %             
                     if (GLOBAL_PIHA.Locations{srcloc}.transitions{m}.clock == i)
                        
                        % -- Compute location number of the destination (empty if terminal) -- % 
                            destination_sfm				= GLOBAL_PIHA.Locations{srcloc}.transitions{m}.destination;
						    [dstloc,destin_location]	= find_location(srcloc,destination_sfm,m);
                        
                        if isempty(dstloc)
                           a=1;
                        end
                      
                        % -- Test if location is terminal -- %             
                        if ~isempty(dstloc)
                           
	                     % -- Apply reset to the reachable set-- %             
                           mapping_pos_reset 	= apply_reset(mapping,GLOBAL_PIHA.Locations{srcloc}.transitions{m}.destination...
                              ,GLOBAL_PIHA.Locations{srcloc}.transitions{m});
                           if PLOT
                               plot(mapping_pos_reset,[1 0 0]);
                           end
                        
		                     % -- cycle through all the interior cells for the location of the destination -- %             
                        	for d=1:length(GLOBAL_PIHA.Locations{dstloc}.interior_cells)
                              
                              cell_number = GLOBAL_PIHA.Locations{dstloc}.interior_cells(d);
                              
				                  % -- Test if mapping intersect given cell of the destination -- %             
                              if ~isempty(and(mapping_pos_reset,return_invariant(cell_number)))
                                 
                               	dst_struc{mapping_counter}.dstloc  	= dstloc;
	                           	dst_struc{mapping_counter}.mapping	= and(mapping_pos_reset,return_invariant(cell_number));
   	                        	dst_struc{mapping_counter}.dstcell 	= cell_number;
      	                  		dst_struc{mapping_counter}.Tstamp   = [inf_pointer(i) sup_pointer(i)];
                                dst_struc{mapping_counter}.transition_theta=0;
                                 if ~strcmp(overall_dynamics,'clock')
                                     %Only perform bounding box if there is more than one mapping
                                     %('if then' added by JPK 1/30/02)
                                     if length(dst_struc{mapping_counter}.mapping)>1
                                        mapping_pos_bb	= bounding_SD_box(dst_struc{mapping_counter},param_file);   
                                    else
                                        mapping_pos_bb	=   dst_struc{mapping_counter}.mapping;
                                    end    
                                else
                                    mapping_pos_bb	=   dst_struc{mapping_counter}.mapping;   
                                 end
            	               	dst_struc{mapping_counter}.mapping	= mapping_pos_bb;
               	            	mapping_counter = mapping_counter + 1;
                  	         end %if%
                           end %for%
                        else
                           %Else add to terminal list if the terminal state has not already been added.
                           found_terminal=0;
                           for q=1:length(terminal)
                               if terminal{q}==destination_sfm
                                   found_terminal=1;
                               end
                           end   
                           if ~found_terminal
                                terminal{length(terminal)+1} = destination_sfm;
                           end
                           
                        end %if%
                  	end%if%
                  	transition_found =1;
                  	temp = minus(Pk_reach,mapping);
          				if ~isempty(temp)
            				Pk_reach  = temp{1};
         				else
            				Pk_reach = temp;
         				end
                       
						end	%if%
		     		end	%for%        
				end	%for%
			end	%if%
      end	%if%
      
      % If guard violated, collect information about the cells in the source location
      if transition_found |(initial & ~isempty(Pk_reach))
         
         % If reachable set is not empty
         if ~isempty(Pk_reach)
            
            % If reachable set went out original cell
            if ~isempty(minus(Pk_reach,return_invariant(srccell)))
               
               %--- test all cells ---% 
               for m=1:length(GLOBAL_PIHA.Locations{srcloc}.interior_cells) 
                  
                  % If reachable set intersects some cell
                  if ~isempty(and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.interior_cells(m))))
                     
                     % create an element of destination to be returned
                     dst_struc{mapping_counter}.mapping   = ...
                    		and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.interior_cells(i)));
		            	dst_struc{mapping_counter}.dstloc   = srcloc;
      	         	dst_struc{mapping_counter}.dstcell  = GLOBAL_PIHA.Locations{srcloc}.interior_cells(m);
           		   	dst_struc{mapping_counter}.Tstamp   = [inf_pointer(i) sup_pointer(i)];
                    dst_struc{mapping_counter}.transition_theta=0;
                     temp = minus(Pk_reach,dst_struc{mapping_counter}.mapping);
         				if ~isempty(temp)
				            Pk_reach  = temp{1};
				         else
				            Pk_reach = temp;
				         end
							mapping_counter = mapping_counter + 1;
                       
                 	end %If%
                    
                end%for%
                
             end%if%
             
          end%if%
          
       end%if%
       
      % Transition for the initial step
      if initial & ~isempty(Pk_reach)
         dst_struc{mapping_counter}.mapping   = and(Pk_reach,return_invariant(srccell));
         dst_struc{mapping_counter}.dstloc   = srcloc;
         dst_struc{mapping_counter}.dstcell  = srccell;
         dst_struc{mapping_counter}.Tstamp   = [inf_pointer(i) sup_pointer(i)];
         dst_struc{mapping_counter}.transition_theta=1;
         temp = minus(Pk_reach,dst_struc{mapping_counter}.mapping);
         if ~isempty(temp)
            Pk_reach  = temp{1};
         else
            Pk_reach = temp;
         end
         dst_struc{mapping_counter}.transition_index   = [];
     		mapping_counter = mapping_counter + 1;
   	  end
      
      if isempty(Pk_reach)|initial
         END_SEARCH = 1;
      end
      
      % check if equilibrium point is inside mapping and if the equilibrium ellipsoid 
      % will prevent the system to make further transitions
   	if check_equilibrium
      	Vk = vertices(Pk_reach);
			% Translate the vertices of the reachable set Xk at time tk from xe
			% to the origin.
			Vk_hat = transform(Vk,eye(size(A)),-xe);
			% Then check if Vk_hat is in the stability ellipsoid centered at the
			% origin.
			equilibrium = is_in_stability_ellipsoid(Vk_hat,Q,gamma);
		end
         
      % check time limit  
   	if check_time_limit
         time_limit = (inf_pointer(i) > max_time);
		end
      
      if time_limit | equilibrium
         END_SEARCH = 1;
      end
      
      % Test if reachable set is out of bound
      if ~isempty(Pk_reach)
         if ~isempty(minus(Pk_reach,AR))
         	if isempty(and(Pk_reach,AR))
            	END_SEARCH = 1;
         	end
         	out_of_bound=1;
         	Pk_reach = and(Pk_reach,AR);
      	end
      end
      counter = counter + 1;
		if (rem(counter-1,60) == 0)
   		fprintf(1,'\n')
		end
   	fprintf(1,'.')
		fprintf(1,'\n')
     	null_event = equilibrium; 
              
  end
    
  destination ={};
  for j=1:length(dst_struc)
     if ~isempty(dst_struc{j}.mapping)
        if ~isempty(minus(dst_struc{j}.mapping,AR))
            out_of_bound=1;
        else
            destination{length(destination)+1}=dst_struc{j};
        end
    end
  end
     
end

%Put a bounding box around destination regions that land in the same destination location
%and cell.
destination = bound_and_clean(destination);


if PLOT
  %close(fig)
end

return

% -----------------------------------------------------------------------------

function destination_new = bound_and_clean(destination_old)

%Put a bounding box around all destination regions that land in the same location and state.
if ~isempty(destination_old)
    unique_dest_list = [destination_old{1}.dstloc destination_old{1}.dstcell];
    %Make a list of all unique destination locations/states.
    for i = 1:length(destination_old)
        if ~ismember([destination_old{i}.dstloc destination_old{i}.dstcell],unique_dest_list,'rows')
            unique_dest_list = [ unique_dest_list ; destination_old{i}.dstloc destination_old{i}.dstcell];
        end
    end
    
    %Use list of unique destinations to create new destination list.
    for i = 1:size(unique_dest_list,1)
        destination_temp={};
        Tstamp=[Inf 0];
        verts=vertices;
        for j = 1:length(destination_old)
            if (destination_old{j}.dstloc==unique_dest_list(i,1))&(destination_old{j}.dstcell==unique_dest_list(i,2))
                    destination_temp{length(destination_temp)+1} = destination_old{j};
                    if destination_old{j}.Tstamp(1)<Tstamp(1)
                        Tstamp(1) = destination_old{j}.Tstamp(1);
                    end 
                    if destination_old{j}.Tstamp(2)>Tstamp(2)
                        Tstamp(2) = destination_old{j}.Tstamp(2);
                    end
                    verts = verts|vertices(destination_old{j}.mapping);
            end
        end
        if length(destination_temp)>1 
        	destination_new{length(destination_new)+1} = destination_temp{1}
            destination_new{length(destination_new)}.Tstamp = Tstamp;  
            mapping_temp =  linearcon(polyhedron(verts));
            destination_new{length(destination_new)}.mapping = return_cell_invariant(unique_dest_list(i,2))&mapping_temp;
        else
            destination_new{length(destination_new)+1} = destination_temp{1};
        end        
    end
end

return

% -----------------------------------------------------------------------------

function result = is_in_stability_ellipsoid(V,Q,gamma)

% Check if the given polytope P defined by a set of vertices V is contained
% in the ellipsoid x'*Q*x <= gamma. This done by solving the quadratic
% program
%
%    max    x'*Q*x 
%  x in P 
%
% and check if the maximum is <= gamma. Solve the maximization problem by
% searching over the vertices V of P, since the global maximum of a convex
% function over a polytope occurs at some vertex of the polytope.

result = 1;
for k = 1:length(V)
  if (V(k)'*Q*V(k) > gamma)
    result = 0;
    break;
  end
end

% -----------------------------------------------------------------------------

function [P,gamma] = lyapell(A,C,d)

% Given a STABLE matrix A and an invariant polytope Cx <= d which strictly
% encloses the origin, find the largest Lyapunov ellipsoid x'*P*x = gamma
% contained in Cx <= d.

% If A is STABLE then, we must be able to solve the Lyapunov equation
P = lyap(A',eye(size(A)));

% To fit the largest ellipsoid x'*P*x, inside the polytope CI*x <= dI which
% stictly encloses the origin, we do the following.
% The largest ellipsoid that could fit inside the kth face of the polytope
% can be found by solving the optimization problem
% 
% min        fk(x) = x'*P*x 
% subject to ck'*x = dk
%
% By writing the Lagrangian L(x,lambda) = x'*P*x + lambda(ck'*x-dk), and
% differentiate with respect to x and lambda, we have that the optimal
% solution occurs at
%
% x = -(lambda/2)*P^{-1}*ck        .............. (1)
%
% From the constraint ck'*x = dk, we have that
%
% dk = -(lambda/2)*ck'*P^{-1}*ck   .............. (2)
%
% Solving (2) for lambda and substituting (1) into the objective
% function, we have that the optimal value for the objective function is
% 
% fk(x) = (dk^2)/(ck'*P^{-1}*ck)
%
% Thus, the largest ellipsoid that contained in the polytope is given by
% x'*P*x = gamma where
%
% gamma = min { (dk^2)/(ck'*P^{-1}*ck) }
%          k

P_inv = inv(P);
gamma = Inf;
for k = 1:length(d)
  gamma_k = (d(k)*d(k))/(C(k,:)*P_inv*C(k,:)');
  if gamma_k < gamma
    gamma = gamma_k;
  end
end

%-----------------------------------------------------------------------------

function [check_equilibrium,check_time_limit,xe,Q,gamma]= fcheck_equilibrium(A,b,INV)

Q=eye(size(A));
gamma = zeros(size(A,1),1);
xe=zeros(size(A,1),1);
% If A is invertible, precompute inverse of A
if rank(A) == size(A,1)
  Ainv = inv(A);
else
  Ainv = [];
end

check_equilibrium =0;
check_time_limit = 0;

if all(real(eig(A)) < 0)
  % A is stable
  [CE,dE,CI,dI] = linearcon_data(INV);
  if ~(isempty(CE) & isempty(dE))
    INV
    error('Invariant polytope must be full dimensional.')
  end
  % The equilibrium point is xe = -Ainv*b.
  xe = -Ainv*b;
  % Translate the invariant from the equilibrium point to the origin
  dIhat = dI-CI*xe;
  % check equilibrium point enclosure
  if all(dIhat > 0)
    % Find the largest stability ellipsoid x'*Q*x = gamma
    % contained in the translated INV.
    [Q,gamma] = lyapell(A,CI,dIhat);
    check_equilibrium = 1;
  end
else 
  if any(real(eig(A)) == 0) & all(b == 0)
    % A is marginally stable and no input
    check_time_limit = 1;
  end
end
