function [destination,null_event,time_limit,out_of_bound,terminal] = ...
    compute_mapping_DHA(X0,sttype,Tstamp,srcloc,srccell)

% Compute the `mapping set` from the given polytope in the specified location.
%
% Syntax:
%   "[destination,null_event,time_limit,out_of_bound,terminal] = ...
%			compute_mapping_DHA(X0,sttype,Tstamp,srcloc,srccell,param_file)"
%
% Description:
%
%   The routine compute_mapping_DHA performs the reachability from a initial set X0, finding  the 
%   set of states that can be reached, using sampled-data analysis (i.e. discrete-time, difference
%   equations with fixed sample periods). It also finds a conservative approximation of the elapsed
%   time to reach them. 
%
%	 After this, it applies the reset functivon defined for this transition. As a last
%   operation, it wraps up the set of states with a conservative bounding box, avoiding explosion 
%   in the number of vertices and faces. 
%
%   The inputs to this function are: 
%
%   * "X0": a "linearcon" object representing the initial continuous set
%     for which the mapping is to be computed.
%  
%   * "sttype": type of state for "X0" which is either an `initial` or
%     `face` state (apply the initial displacement or not).
%
%	 * "Tstamp": vector with the time range in which the initial set occurs.
%       Note that if the sample rate is deterministic, then this is a vector
%       with two identical elements.
% 
%   * "srcloc": source location.
% 
%	 * "srccell": cell region of source
%
%
%   The outputs of this function are
%
%   % `destination is a cell array with the following fields:
%
%   * dstloc 	: location whose guard was violated during the reachability process
%	 * dstcell 	: Interior region of the dstloc
%   * region 	: linearcon object representing the set of states that violates some guard for a given
%     location.
%   * Tstamp 	: time elapsed interval to reach the region starting at the set X0.
%
%   * "null_event": a boolean flag indicating that the flow pipe computation
%     was terminated because it can be concluded that the subsequent flow
%     pipe segments will remain inside "INV" forever.
%
%   * "time_limit": a boolean flag indicating that the flow pipe computation
%     was terminated because the time limit "max_time" was exceeded.
%
% Implementation:
%   Call the mapping computation routine corresponding to the type of the
%   composite continuous dynamics of all SCSBs for the given location
%   ("fs_lin_map" for `linear` (affine) dynamics, "fs_nonlin_map" for
%   `nonlinear` dynamics, and "clk_map" for `clock` dynamics). %   
%
% See Also:
%   fs_nonlin_map,fs_lin_map,clk_map

global GLOBAL_PIHA
global GLOBAL_APPROX_PARAM
PLOT=0;
% Get the invariant polytope for the given cell location.
INV = return_invariant(srccell);

% Compute the type of the composite dynamics of all SCSBs for the FSM
% state in the source location.

overall_dynamics = check_overall_dynamics(GLOBAL_PIHA.SCSBlocks, ...
    GLOBAL_PIHA.Locations{srcloc}.q);

max_time = GLOBAL_APPROX_PARAM.max_time;

% Get clock structure information
clock_info = GLOBAL_PIHA.CLOCKBlocks;

null_event = 0;
time_limit = 0;
terminal = {};
out_of_bound = 0;

% inf_pointer 	             = ones(length(clock_info),1)*Tstamp(1);
% sup_pointer 	            = ones(length(clock_info),1)*Tstamp(2);
% inf_pointer_nojitter 	   = ones(length(clock_info),1)*Tstamp(1);
% sup_pointer_nojitter 	  = ones(length(clock_info),1)*Tstamp(2);

%inf_pointer 	             = zeros(length(clock_info),1);
%sup_pointer 	            = zeros(length(clock_info),1);
%inf_pointer_nojitter 	   = zeros(length(clock_info),1);
%sup_pointer_nojitter 	  = zeros(length(clock_info),1);

equilibrium=0;
check_equilibrium=0;
check_time_limit=1;

% Get information about analysis region
for k=1:GLOBAL_PIHA.NAR
    c_AR(k,:) = GLOBAL_PIHA.Hyperplanes{k}.c;
    d_AR(k,:) = GLOBAL_PIHA.Hyperplanes{k}.d;
end
AR = linearcon([],[],c_AR,d_AR);
% if PLOT
%     plot(AR,[1 0 0]);
% end

fprintf(1,'Computing flow pipe segments:')
counter = 0;

global_var;
%PLOT = 0;

switch sttype
    
    case 'init', % initial state
        initial = 1;
        
    case 'face', % face state (a patch)
        initial=0;
        
    otherwise,
        error(['Unknown state type ''' sttype '''.'])
        return
end



%equilibrium = 0; time_limit = 0;Xsim = vertices;
%first = 1; t_total = 0; stop = 0;init_T=T;


if strcmp(overall_dynamics,'linear')
    %Get info for linear case and find a lyapunov ellipsoid if one exists within the interior
    %cells of this location
    
    %Currently, DHA analysis only supports one clock, one SCSB block, and one Stateflow chart.
    swfunc = GLOBAL_PIHA.SCSBlocks{1}.swfunc;
    nx = GLOBAL_PIHA.SCSBlocks{1}.nx;
    nz = GLOBAL_PIHA.SCSBlocks{1}.nz;
    nup = GLOBAL_PIHA.SCSBlocks{1}.nup;
    nu = GLOBAL_PIHA.SCSBlocks{1}.nu;
    u = GLOBAL_PIHA.Locations{srcloc}.q;
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % % Commented by Smriti Feb 13                
    
    %if inf_pointer(1)~=sup_pointer(1)|length(inf_pointer)>1
    %    error('Currently, DHA analysis only supports one clock with deterministic behavior.')
    %end
    
    % END OF COMMENT
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %Time varying system case:
    %[sys,type] = feval(swfunc,zeros(nx+nz+nup,1),u,inf_pointer(1));
    
    [sys,type] = feval(swfunc,zeros(nx+nz+nup,1),u);
    A = sys.A;
    b = sys.b;
    
    % Commented out - Smriti - Feb 13
    %                 r = sys.r;
    %                 C = sys.C;
    %                 Ap = sys.Ap;
    %                 bp = sys.bp;
    %                 rp = sys.rp;
    %                 Cp = sys.Cp;
    %                 Dp = sys.Dp;
    %                 rpp = sys.rpp;
    % END COMMENT
    
    
    % If A is invertible, precompute inverse of A
    if rank(A) == size(A,1)
        Ainv = inv(A);
    else
        Ainv = [];
    end
    
    %The value if the plant states at the next sample instant as well as the value of the 
    %controller states at the next sample instant at calculated in one step.
    %The result of this mapping is then used to test the guard conditions, and so
    %the guard conditions are testing x(k+1) and z(k+1).  If a guard is satisfied,
    %the mode is switched (and a reset is applied to the z(k+1) value of the controller state if 
    %need be), and the control law for the new mode is used to then calculate 
    %x(k+2) and z(k+2), and so on.
    %
    % SG % Feb 13 %               T=clock_info{1}.period(1);
    T=GLOBAL_PIHA.sample_time;
    %Also note that, currently, only completely deterministic clocks are allowed.
    %If the transformation is given by Pk_reach=A*Pk_reach+B then the following
    %two lines compute A and B respectively (based on the linear dynamics specified
    %above).
    I=eye(size(A,1));
    
    % Commented out - Smriti - Feb 13
    
    %                 if ~isempty(Ainv)
    %                     t = [ expm(A*T)+(expm(A*T)-I)*Ainv*b*Dp*C (expm(A*T)-I)*Ainv*b*Cp ;
    %                                    bp*C                            Ap ];
    %                     v = [ (expm(A*T)-I)*Ainv*(b*rpp+r) ;
    %                                    rp ];
    %                 else
    %                     %If the inverse of A does not exist, then, in the following equation
    %                     %the intergral must be calculated numerically by way of the 'step_response' procedure:
    %                     %
    %                     %   x(k+1) = e^(AT)*k(k) + integral(e^(A*(T-tau))dtau * (B*u(k)+r)
    %                     %
    %                     for col_idx=1:size(A,1)
    %                         temp = zeros(length(b),1);
    %                         temp(col_idx)=1;
    %                         integral(:,col_idx) = step_response(A,[],temp,T);                   
    %                     end
    %                     
    %                     t = [ expm(A*T)+integral*b*Dp*C integral*b*Cp ;
    %                                    bp*C                    Ap ];
    %                      v = [ integral*(b*rpp+r) ;
    %                                    rp ];
    %                     
    %                 end
    % END COMMENT
    
    
    % Added by Smriti - Feb 13
    t = A;
    v = b;
    
    %Find a lyapunov ellipsoid if one exists within any of the cells that make
    %up this location's interior
    for i=GLOBAL_PIHA.Locations{srcloc}.interior_cells
        INV_temp = return_cell_invariant(i);
        [check_equilibrium,check_time_limit,xe,Q,gamma]= fcheck_equilibrium(t,v,INV_temp);
        if check_equilibrium==1
            break;
        end
    end
    
    
    
end


% Compute the transformation of the initial segment

time_pointer = 0;
mapping_counter =1;
dst_struc = {};

% cycle through the clock structure items and  identifying the possible transitions
% for i=1:length(clock_info)
Pk_reach = X0;
END_SEARCH = 0;   
% SG % Feb 2004 - changed %      Tinterval          = sup_pointer(i) - inf_pointer(i);
Tinterval = GLOBAL_PIHA.sample_time;
% added by xiyan 03/11/2004
Tstamp_temp = Tstamp;

figure(1);
plot(Pk_reach);
hold on;
while ~END_SEARCH
    transition_found=0;
        
    if ~isempty(Pk_reach)
        switch overall_dynamics
            
            case 'linear'
                
                %If this is the first step, do not perform the transformation.
                %Just test the guard to see if it is satisfied.  This implies
                %that the system does not need to remain in the first state for one
                %sample period, it can switch modes at the first instant (i.e. t=0).
                if initial~=1                  
                    Pk_reach = transform(Pk_reach,t,v);
                    [cetest,detest,citest,ditest] = linearcon_data(Pk_reach);
                    if length(detest)>0
                        
                        %changed by xiyan
                        Pk_reach = grow_polytope(cetest,detest,citest,ditest,[0 0 0]);    
                    end
                    
                end
                figure(1)
                plot(Pk_reach);
            case 'clock'
                
                v = overall_system_clock(GLOBAL_PIHA.SCSBlocks, ...
                    GLOBAL_PIHA.Locations{srcloc}.q); 
                
                Pinit  	= transform(Pk_reach,eye(length(v)),v*inf_pointer(i));
                Pfinal 	= transform(Pk_reach,eye(length(v)),v*sup_pointer(i));
                Xinit 	=	vertices(Pinit);
                Xfinal 	=	vertices(Pfinal);
                Pk			= polyhedron(Xinit | Xfinal);
                Pk_reach	= linearcon(Pk);
                
            case 'nonlinear'
                
                if initial~=1   
                    ode_param = GLOBAL_PIHA.Locations{srcloc}.q;
                    check_time_limit = 1;
                    
                    sample_points_X0 = vertices(Pk_reach);
                    Pk = linearcon(Pk_reach);
                    %Only deterministic sample periods are currently supported.
                    T=clock_info{i}.period(1);
                    [Pk,sample_points] = seg_approx_DHA_ode('overall_system_ode_for_DHA',ode_param,...
                        Pk,sample_points_X0,inf_pointer(i),[],T);
                    Pk_reach = Pk;
                end           
                
            otherwise,
                error(['Unknown dynamics type ''' overall_dynamics '''.'])       
                
        end 	%switch%
        if 0        
            drawnow;plot(clean_up(project(Pk_reach,[1 2])),[0 0 0]);  
        end
        
    end	%if%
    
    
    %% switch statement commented out by SG (Feb 2004)
    % ----- Advance one step of clock  ---------% 
    %       switch initial
    %     
    %    		case 1
    %    	   	 % Compute the reachable set for the first sampling
    %       	     inf_pointer_nojitter(i) = clock_info{i}.phase(1);
    %          	 sup_pointer_nojitter(i) = clock_info{i}.phase(2);
    %        	     inf_pointer(i) = inf_pointer_nojitter(i)  + clock_info{i}.jitter(1);
    %          	 sup_pointer(i) = sup_pointer_nojitter(i) +clock_info{i}.jitter(2);
    % 
    %           
    % 	   	case 0
    %    	      % Compute the reachable set for the other sampling
    %            inf_pointer_nojitter(i) =inf_pointer_nojitter(i) + clock_info{i}.period(1) ;
    %            sup_pointer_nojitter(i) =sup_pointer_nojitter(i) + clock_info{i}.period(2) ;
    %       	 	inf_pointer(i) = inf_pointer_nojitter(i) +clock_info{i}.jitter(1);
    %        		sup_pointer(i)	= sup_pointer_nojitter(i) + clock_info{i}.jitter(2);
    %    	  end
    
    
    if (Tinterval < 0)
        error(['Inconsistent value of clock = [''' Tinterval '''].'])
    end
    
    
    if ~isempty(Pk_reach)
        
        % -- test if Pk_reach left cell -- %
        if ~isempty(minus(Pk_reach,return_invariant(srccell)))
            
            % -- Cycle through possible transitions -- %             
            for m=1:length(GLOBAL_PIHA.Locations{srcloc}.transitions)
                
                % -- Cycle through each guard cell for each possible transition -- %             
                for k=1:length(GLOBAL_PIHA.Locations{srcloc}.transitions{m}.guard)
                    
                    % -- Test if guard cell was reached by Pk_reach -- %             
                    if ~isempty(and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.transitions{m}.guard(k))))
                        
                        % -- compute intersection between Pk_reach and the guard cell -- %             
                        mapping  = and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations...
                            {srcloc}.transitions{m}.guard(k)));
                        
                        % -- Test if transitions refers to clock used in this loop -- %           % This loop is suspect! -Smriti  
                        % if (GLOBAL_PIHA.Locations{srcloc}.transitions{m}.clock == i)
                            
                            % -- Compute location number of the destination (empty if terminal) -- % 
                            destination_sfm				= GLOBAL_PIHA.Locations{srcloc}.transitions{m}.destination;
                            
                            % Changed by Xiyan and Jim K (4/04) to stop if
                            % the reachability hit an "Avoid" state.
                            name_dummy = GLOBAL_PIHA.Locations{srcloc}.transitions{m}.destination_name;
                            if findstr('avoid',name_dummy)
                                global STOP_VERIFICATION_FLAG
                                STOP_VERIFICATION_FLAG = name_dummy;
                            end
                            
                            [dstloc,destin_location]	= find_location(srcloc,destination_sfm,m);
                            
                            if isempty(dstloc)
                                a=1;
                            end
                            
                            % -- Test if location is terminal -- %             
                            if ~isempty(dstloc)
                                
                                % -- Apply reset to the reachable set-- %             
                                mapping_pos_reset 	= apply_reset(mapping,GLOBAL_PIHA.Locations{srcloc}.transitions{m}.destination...
                                    ,GLOBAL_PIHA.Locations{srcloc}.transitions{m});
                                if PLOT
                                    plot(mapping_pos_reset,[1 0 0]);
                                end
                                
                                % -- cycle through all the interior cells for the location of the destination -- %             
                                for d=1:length(GLOBAL_PIHA.Locations{dstloc}.interior_cells)
                                    
                                    cell_number = GLOBAL_PIHA.Locations{dstloc}.interior_cells(d);
                                    
                                    % -- Test if mapping intersect given cell of the destination -- %             
                                    if ~isempty(and(mapping_pos_reset,return_invariant(cell_number)))
                                        
                                        dst_struc{mapping_counter}.dstloc  	= dstloc;
                                        dst_struc{mapping_counter}.mapping	= and(mapping_pos_reset,return_invariant(cell_number));
                                        dst_struc{mapping_counter}.dstcell 	= cell_number;
                                        
                                        % SG % Feb 2004% changed
                                        %Tstamp_temp = Tstamp + GLOBAL_PIHA.sample_time;
                                        %dst_struc{mapping_counter}.Tstamp   = Tstamp_temp;
                                        
                                        %changed by xiyan 03/09
                                        dst_struc{mapping_counter}.Tstamp   = Tstamp_temp + Tinterval;
                                        % change DONE
                                        
                                        dst_struc{mapping_counter}.transition_theta=0;
                                        if ~strcmp(overall_dynamics,'clock')
                                            %Only perform bounding box if there is more than one mapping
                                            %('if then' added by JPK 1/30/02)
                                            if length(dst_struc{mapping_counter}.mapping)>1
                                                mapping_pos_bb	= bounding_SD_box(dst_struc{mapping_counter},param_file);   
                                            else
                                                mapping_pos_bb	=   dst_struc{mapping_counter}.mapping;
                                            end    
                                        else
                                            mapping_pos_bb	=   dst_struc{mapping_counter}.mapping;   
                                        end
                                        dst_struc{mapping_counter}.mapping	= mapping_pos_bb;
                                        mapping_counter = mapping_counter + 1;
                                    end %if%
                                end %for%
                            else
                                %Else add to terminal list if the terminal state has not already been added.
                                found_terminal=0;
                                for q=1:length(terminal)
                                    if terminal{q}==destination_sfm
                                        found_terminal=1;
                                    end
                                end   
                                if ~found_terminal
                                    terminal{length(terminal)+1} = destination_sfm;
                                end
                                
                            end %if%
                            %                        end%if%
                        transition_found =1;
                        
                        % The following lines were commented out (JPK 6/2002) because there is no apparent reason why they
                        % should be here.  Izaias apparently wanted to subtract the overlap of the guard cell with the reachable
                        % set from the entire reachable set (then intersect with the next guard and then intersect withe the other
                        % interior cells).  I don't think this is necessary or even correct.  For one thing, the guards are not necessarily
                        % disjoint.  For another thing, the subtraction operation will yield multiple linearcons as a result if the result
                        % is nonconvex.
                        %
                        %                   	temp = minus(Pk_reach,mapping);
                        %           				if ~isempty(temp)
                        %             				Pk_reach  = temp{1};
                        %          				else
                        %             				Pk_reach = temp;
                        %          				end
                        
                    end	%if%
                end	%for%        
            end	%for%
        end	%if%
    end	%if%
    
    % If guard violated, collect information about the cells in the source location
    if transition_found |(initial & ~isempty(Pk_reach))
        
        % If reachable set is not empty
        if ~isempty(Pk_reach)
            
            % If reachable set went out original cell
            if ~isempty(minus(Pk_reach,return_invariant(srccell)))
                
                %--- test all cells ---% 
                for m=1:length(GLOBAL_PIHA.Locations{srcloc}.interior_cells) 
                    
                    % If reachable set intersects some cell
                    if (srccell~=GLOBAL_PIHA.Locations{srcloc}.interior_cells(m))&...
                            ~isempty(and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.interior_cells(m))))
                        
                        % create an element of destination to be returned
                        dst_struc{mapping_counter}.mapping   = ...
                            and(Pk_reach,return_invariant(GLOBAL_PIHA.Locations{srcloc}.interior_cells(m)));
                        dst_struc{mapping_counter}.dstloc   = srcloc;
                        dst_struc{mapping_counter}.dstcell  = GLOBAL_PIHA.Locations{srcloc}.interior_cells(m);
                        
                        % added by xiyan
                        dst_struc{mapping_counter}.Tstamp   = Tstamp_temp + Tinterval;
                        %dst_struc{mapping_counter}.Tstamp   = [inf_pointer(i) sup_pointer(i)];
                        
                        dst_struc{mapping_counter}.transition_theta=0;
                        
                        % Again, I don't think it's correct to perform the set subtraction here.  We want to intersect the reachable set 
                        % with all of the interior cells, but we can do that with the entire reachable set (JPK 6/2002)
                        %                                          
                        %                      temp = minus(Pk_reach,dst_struc{mapping_counter}.mapping);
                        %          				if ~isempty(temp)
                        % 				            Pk_reach  = temp{1};
                        % 				         else
                        % 				            Pk_reach = temp;
                        % 				         end
                        
                        mapping_counter = mapping_counter + 1;
                        
                    end %If%
                    
                end%for%
                
            end%if%
            
        end%if%
        
    end%if%
    
    % Transition for the initial step
    if initial & ~isempty(Pk_reach)
        dst_struc{mapping_counter}.mapping   = and(Pk_reach,return_invariant(srccell));
        dst_struc{mapping_counter}.dstloc   = srcloc;
        dst_struc{mapping_counter}.dstcell  = srccell;
        
        % SG % Feb 2004% changed
        %Tstamp_temp = Tstamp + GLOBAL_PIHA.sample_time;
        %dst_struc{mapping_counter}.Tstamp   = Tstamp_temp;
        
        %changed by xiyan 03/09/2004
        dst_struc{mapping_counter}.Tstamp   = Tstamp_temp + Tinterval;
        %dst_struc{mapping_counter}.Tstamp   = [inf_pointer(i) sup_pointer(i)];
        % Change DONE
        
        dst_struc{mapping_counter}.transition_theta=1;
        
        % Once again, I don't think that this set subtraction is appropriate (JPK 6/2002)
        %          
        %          temp = minus(Pk_reach,dst_struc{mapping_counter}.mapping);
        %          if ~isempty(temp)
        %             Pk_reach  = temp{1};
        %          else
        %             Pk_reach = temp;
        %          end
        dst_struc{mapping_counter}.transition_index   = [];
        mapping_counter = mapping_counter + 1;
    end
    
    
    
    % check if equilibrium point is inside mapping and if the equilibrium ellipsoid 
    % will prevent the system to make further transitions
    if check_equilibrium
        Vk = vertices(Pk_reach);
        % Translate the vertices of the reachable set Xk at time tk from xe
        % to the origin.
        Vk_hat = transform(Vk,eye(size(A)),-xe);
        % Then check if Vk_hat is in the stability ellipsoid centered at the
        % origin.
        equilibrium = is_in_stability_ellipsoid(Vk_hat,Q,gamma);
    end
    
    % check time limit  
    if check_time_limit
        % changed by xiyan
        %time_limit = (inf_pointer(i) > max_time);
        time_limit = (Tstamp_temp(1) > max_time);
    end
    
    if time_limit | equilibrium
        END_SEARCH = 1;
    end
    
    % Test if reachable set is out of bound
    if ~isempty(Pk_reach)
        if ~isempty(minus(Pk_reach,AR))
            if isempty(and(Pk_reach,AR))
                END_SEARCH = 1;
            end
            out_of_bound=1;
        end
    end
    
    %After intersecting Pk_reach with all possible guards, all possible other interior regions,
    %and the complement of the analysis region, all that remains is the part ofthe reachable set 
    %that is still in the original source cell.  So the following line was added (JPK 6/2002).
    %This should be done only if a transition was found
    %because, otherwise we don't require that the reachable set remain within one interior cell.
    if transition_found
        Pk_reach = and(Pk_reach,return_invariant(srccell));
    end
    
    if isempty(Pk_reach)|initial
        END_SEARCH = 1;
    end
    
    counter = counter + 1;
    if (rem(counter-1,60) == 0)
        fprintf(1,'\n')
    end
    fprintf(1,'.')
    fprintf(1,'\n')
    null_event = equilibrium; 
    
    % added by xiyan 03/11/2004, for the computation of Tstamp of the
    % pure discrete-time systems(fixed rate)
    Tstamp_temp = Tstamp_temp + Tinterval;
    
end

destination ={};
for j=1:length(dst_struc)
    if ~isempty(dst_struc{j}.mapping)
        if ~isempty(minus(dst_struc{j}.mapping,AR))
            out_of_bound=1;
        else
            destination{length(destination)+1}=dst_struc{j};
        end
    end
end

%end

%Put a bounding box around destination regions that land in the same destination location
%and cell.

%The 'bound_and_clean' call has been commented out because we want the mappings to remain separated
%so that each one will have its respective time tag.  This is necessary now because the system
%dynamics are time-varying.

%destination = bound_and_clean(destination);


if PLOT
    %close(fig)
end

return

% -----------------------------------------------------------------------------

function destination_new = bound_and_clean(destination_old)

%Put a bounding box around all destination regions that land in the same location and state.

destination_new={};

if ~isempty(destination_old)
    unique_dest_list = [destination_old{1}.dstloc destination_old{1}.dstcell];
    %Make a list of all unique destination locations/states.
    for i = 1:length(destination_old)
        if ~ismember([destination_old{i}.dstloc destination_old{i}.dstcell],unique_dest_list,'rows')
            unique_dest_list = [ unique_dest_list ; destination_old{i}.dstloc destination_old{i}.dstcell];
        end
    end
    
    
    %Use list of unique destinations to create new destination list.
    for i = 1:size(unique_dest_list,1)
        destination_temp={};
        Tstamp_bound=[Inf 0];
        verts=vertices;
        for j = 1:length(destination_old)
            if (destination_old{j}.dstloc==unique_dest_list(i,1))&(destination_old{j}.dstcell==unique_dest_list(i,2))
                destination_temp{length(destination_temp)+1} = destination_old{j};
                if destination_old{j}.Tstamp(1)<Tstamp_bound(1)
                    Tstamp_bound(1) = destination_old{j}.Tstamp(1);
                end 
                if destination_old{j}.Tstamp(2)>Tstamp_bound(2)
                    Tstamp_bound(2) = destination_old{j}.Tstamp(2);
                end
                verts = verts|vertices(destination_old{j}.mapping);
            end
        end
        if length(destination_temp)>1 
            destination_new{length(destination_new)+1} = destination_temp{1};
            destination_new{length(destination_new)}.Tstamp = Tstamp_bound;  
            mapping_temp =  linearcon(polyhedron(verts));
            destination_new{length(destination_new)}.mapping = return_cell_invariant(unique_dest_list(i,2))&mapping_temp;
        else
            destination_new{length(destination_new)+1} = destination_temp{1};
        end        
    end
end

return

% -----------------------------------------------------------------------------

function result = is_in_stability_ellipsoid(V,Q,gamma)

% Check if the given polytope P defined by a set of vertices V is contained
% in the ellipsoid x'*Q*x <= gamma. This done by solving the quadratic
% program
%
%    max    x'*Q*x 
%  x in P 
%
% and check if the maximum is <= gamma. Solve the maximization problem by
% searching over the vertices V of P, since the global maximum of a convex
% function over a polytope occurs at some vertex of the polytope.

result = 1;
for k = 1:length(V)
    if (V(k)'*Q*V(k) > gamma)
        result = 0;
        break;
    end
end

% -----------------------------------------------------------------------------

function [P,gamma] = lyapell(A,C,d)

% Given a STABLE matrix A and an invariant polytope Cx <= d which strictly
% encloses the origin, find the largest Lyapunov ellipsoid x'*P*x = gamma
% contained in Cx <= d.

% If A is STABLE then, we must be able to solve the Lyapunov equation
P = dlyap(A',eye(size(A)));

% To fit the largest ellipsoid x'*P*x, inside the polytope CI*x <= dI which
% stictly encloses the origin, we do the following.
% The largest ellipsoid that could fit inside the kth face of the polytope
% can be found by solving the optimization problem
% 
% min        fk(x) = x'*P*x 
% subject to ck'*x = dk
%
% By writing the Lagrangian L(x,lambda) = x'*P*x + lambda(ck'*x-dk), and
% differentiate with respect to x and lambda, we have that the optimal
% solution occurs at
%
% x = -(lambda/2)*P^{-1}*ck        .............. (1)
%
% From the constraint ck'*x = dk, we have that
%
% dk = -(lambda/2)*ck'*P^{-1}*ck   .............. (2)
%
% Solving (2) for lambda and substituting (1) into the objective
% function, we have that the optimal value for the objective function is
% 
% fk(x) = (dk^2)/(ck'*P^{-1}*ck)
%
% Thus, the largest ellipsoid that contained in the polytope is given by
% x'*P*x = gamma where
%
% gamma = min { (dk^2)/(ck'*P^{-1}*ck) }
%          k

P_inv = inv(P);
gamma = Inf;
for k = 1:length(d)
    gamma_k = (d(k)*d(k))/(C(k,:)*P_inv*C(k,:)');
    if gamma_k < gamma
        gamma = gamma_k;
    end
end

%-----------------------------------------------------------------------------

function [check_equilibrium,check_time_limit,xe,Q,gamma]= fcheck_equilibrium(A,b,INV)

Q=eye(size(A));
gamma = zeros(size(A,1),1);
xe=zeros(size(A,1),1);
% If A is invertible, precompute inverse of A
if rank(eye(size(A))-A) == size(A,1)
    i_minus_A_inv = inv(eye(size(A))-A);
else
    i_minus_A_inv = [];
end

check_equilibrium =0;
check_time_limit = 0;

if all(abs(eig(A))<1)
    % A is stable (in discrete time).
    [CE,dE,CI,dI] = linearcon_data(INV);
    if ~(isempty(CE) & isempty(dE))
        INV
        error('Invariant polytope must be full dimensional.')
    end
    % The equilibrium point (for discrete time) is xe = inv(I-A)*b.
    xe = i_minus_A_inv*b;
    % Translate the invariant from the equilibrium point to the origin
    dIhat = dI-CI*xe;
    % check equilibrium point enclosure
    if all(dIhat > 0)
        % Find the largest stability ellipsoid x'*Q*x = gamma
        % contained in the translated INV.
        [Q,gamma] = lyapell(A,CI,dIhat);
        check_equilibrium = 1;
    end
else 
    %if any(real(eig(A)) == 0) & all(b == 0)
    
    %changed by xiyan 03/11/2004. For discret-time systems, abs(eig(A))==1
    %is the condition of oscillation. I don't check b matrix.
     if any(abs(eig(A)==1))  
        % A is marginally stable and no input
        check_time_limit = 1;
    end
end
