function SEG = seg_approx_lin(A,Ainv,b,X0,SP0,T)

% Approximate a single segment of a flow pipe for a `linear` dynamics.
%
% Syntax:
%   "SEG = seg_approx_lin(A,Ainv,b,X0,SP0,T)"
%
% Description:
%   Compute a conservative approximation of a flow pipe segment in the time
%   interval "[0,T]" for the `linear` (affine) dynamics "dx/dt = A*x +
%   b". The inputs to this function are
%
%   * "A": the system matrix
%
%   * "Ainv": the inverse of "A" if it exists, otherwsie it should be
%     "[]"
%
%   * "b": constant input vector for the affine dynamics
%
%   * "X0": a "linearcon" object represeting the initial set
%
%   * "SP0": sample points on "X0" that are simulated to construct the
%     convex hull. Typically, "SP0" contains the vertices of "X0".
%
%   * "T": the time step for the flow pipe approximation
%
%   The output "SEG" is a "linearcon" object (a polytope) representing the
%   flow pipe segment approximation.
%
% Implementation:
%   To approximate the flow pipe segment, we first simulate the sample
%   points from time "t = 0" to "t = T" and compute the convex hull of the
%   sample points at time "t = 0" and "t = T". For the affine dynamics, the
%   sample points at time "t = T" can be found by the affine transformation
%
%
%
%   "x(T) = e^{A*T}*x(0) + e^{A*T} * integral_{s=0}^{s=T} e^{-A*s}*b ds"
%
%
%
%   The second term, the displacement in the affine transformation, in the
%   above expression is computed by the function "step_response()". The
%   affine transformation performed by calling the function "transform" for
%   the corresponding object type. After the convex hull is obtained, we
%   extract the matrix-vector pair "[CI,dI]" that represents the linear
%   inequality "CI*x <= dI" for the convex hull. The normal vectors for the
%   faces of the conex hull, which are the rows of the matrix "CI", are used
%   to solve the optimization problem.
%
%
%   
%   "max (x0 in X0, t in [0,T]) ci'*x(t,x0)"
%
%
%
%   where "ci" is the "i"-th normal vector and "x(t,x0)" denote the solution
%   to the affine differential equation. We can rewrite the above
%   optimization problem as
%
%
%
%   "max (t in [0,T]) { max (xt in Rt)  ci'*xt }"
%
%
%
%   where "Rt" is the reachable set from "X0" at time "t". For affine
%   dynamics, "Rt" is also a polytope obtained by applying the above affine
%   transformation to the set "X0" with "T" replaced by "t". It is clear
%   that the inner-loop optimization problem is a linear program and the
%   optimal solution can always be found. To increase the chance of finding
%   the global solution, the outer-loop optimization problems are solved
%   with two initial guesses for "t" at "0.01*T" and "0.99*T". The
%   computation of the objective function for the outer-loop optimization
%   problem, which includes solving a linear program is computed in the
%   function "stretch_func_lin()". The outer-loop optimization problem is
%   solved by calling the MATLAB routine for solving constrained nonlinear
%   optimization problem "constr".
%
%
%
%   After solving the optimization problem, we adjust the inequality
%   constraints for the convex hull to be "CI*x <= dInew" where "dInew(i)"
%   is the solution obtained for the "i"-th optimization problem. This, in
%   effect, expands the convex hull to cover the flow pipe segment. Put the
%   new linear inequality constraints into a "linearcon" object, call the
%   function "clean_up()" to remove redundant constraints, and return the
%   result.
%   
%
% See Also:
%   stretch_func_lin,step_response,psim_lin,fs_lin_map,linearcon,transform,
%   clean_up

global GLOBAL_APPROX_PARAM

timing = 0;

eAT = expm(A*T);
displacement = step_response(A,Ainv,b,T);
% Transfrom sample points by eAT
SPf = transform(SP0,eAT,displacement);
Xf = transform(X0,eAT,displacement);
%plot(X0);
%plot(Xf);
% Compute convex hull from these sample points
CH = polyhedron(SP0 | SPf);
[CE,dE,CI,dI] = linearcon_data(linearcon(CH));
t_start = clock;
[CInew,dInew] = shwrap_lin(CI,A,Ainv,b,X0,T);
if timing
  fprintf(1,'time to shrink wrap segment = %f sec\n',etime(clock,t_start))
end
for k = 1:length(dI)
  if (dInew(k) > dI(k))
    dI(k) = dInew(k);
 else
   if (dInew(k) < dI(k) - GLOBAL_APPROX_PARAM.poly_epsilon)
      fprintf(1,['Warning: Optimization result is not reliable, falling' ...
	    ' back to the convex hull approximation\n']);
    end
  end
end
t_start = clock;
SEG = clean_up(linearcon([],[],CI,dI));
if timing
  fprintf(1,'time to clean up = %f sec\n',etime(clock,t_start))
end
return

% -----------------------------------------------------------------------------

function [C,d] = shwrap_lin(C,A,Ainv,b,X0,T)

% Shrink wrap the flow pipe segment in a polytope using the normal vectors
% stored in the rows of the given matrix C, i.e. compute the smallest linear
% constraint set given the set of directions C that contains the flow pipe
% segment from 0 to T.
% C             : a set of directions stacked together in a matrix
% X0            : a constraint representing initial set X0
% A             : system matrix
% Ainv          : inverse of A if exists, otherwise should be []
% b             : constant input vector
% T             : time step with respect to X0
%
% Revised by OS on 02/15/02

global GLOBAL_APPROX_PARAM

calls = GLOBAL_APPROX_PARAM.max_func_calls;
tolerance = GLOBAL_APPROX_PARAM.func_tol;

% Setting some options for MATLAB optimization toolbox
options = optimset('MaxFunEvals',calls*size(C,2),'TolFun',T*tolerance,'Display','off');

% Stretch along each given direction to cover reachability region
d = zeros(size(C,1),1);
for l = 1:size(C,1)
  n_vector = C(l,:);
  topt = []; dl = [];
  
  % Changed by OS: (1) from 'constr' to 'fmincon'
  %                         (2) initialization: instead of calling the nonlinear optimization twice for
  %                              X0=0.01*T and X0=0.99*T now: one optimization for X0=0.5*T
  %                              and comparison with the values for t=0 and t=T
  
  topt=fmincon('stretch_func_lin',0.5*T,[],[],[],[],0,T,[],options,A,Ainv,b,n_vector,X0);   
  dl(1)=-stretch_func_lin(topt,A,Ainv,b,n_vector,X0);
  dl(2)=-stretch_func_lin(0,A,Ainv,b,n_vector,X0);
  dl(3)=-stretch_func_lin(T,A,Ainv,b,n_vector,X0);
  
  [dlmax,idx] = max(dl);
  d(l) = dl(idx);
  
end
return


