function iauto_part()

% Compute initial partition for the initial `approximating automaton`
% according to the user specified parameters.
%
% Syntax:
%   "iauto_part()"
%
% Description:
%   Compute the initial partition for the initial approximating automaton
%   and create the skeleton for the approximating automaton in the global
%   variable "GLOBAL_AUTOMATON".
%
% Implementation:
% 
%   The approximating automaton "GLOBAL_AUTOMATON" is a discretization of
%   "GLOBAL_PIHA" with a few special states added to each
%   location. "GLOBAL_AUTOMATON" is a cell array in which each element
%   corresponds to a location in "GLOBAL_PIHA". Each element
%   "GLOBAL_AUTOMATON{l}" is a structure containing the following fields
%   for the location "l".
%
%   * ".initstate": a cell array of states representing to the
%     discretization of `initial continuous set` for each location
%
%   * ".face": a cell array of the same length as the number of the
%     invariant faces for the location "l"
%
%   The field ".face" is also a structure with a single field ".state" where
%
%   * ".face{f}.state": is a cell array of states representing the
%     discretization of face "f" of the the invariant polytope for location "l"
%
%   States under the ".initstate" field are referred to as `initial states`
%   and states under the ".face" field are referred to as `face
%   states`. Each state is a structure with the following fields.
%
%   * ".polytope": Continuous set for this state.
%
%   * ".mapping": The `mapping set` from the polytope.
%
%   * ".children": Indices to destination `face` states.
%
%   * ".null_event": A boolean flag indicating whether there exists a
%     trajectory from the polytope for this state that never leaves the
%     invariant
%
%   * ".time_limit": A boolean flag indicating wheter the mapping computation
%     is terminated by the user specified time limit in which case the
%     computed mapping set may not contain all possible reachable states.
%
%   * ".out_of_bound": A boolean flag indicating whether part of the mapping
%     computed goes out of analysis region.
%
%   * ".indeterminate": A boolean flag. If set, it indicates that further
%     evolutions of the system from this state cannot be determined. Reasons
%     maybe that polytope for this state contains singularities so that
%     mapping cannot be computed. This flag is typically be set manually by
%     user. CheckMate also set this flag to 1 for those states that are
%     not reachable from the initial states to save computation time.
%
%   * ".terminal": A cell array containing the list of FSM terminal state
%     vectors, i.e. the FSM state for which there is no outgoing transition
%     from any of the component states, that a reachable from the polytope
%     for this state.
%
% Implementation:
%   The polytopes in the initial continuous set are intersected with the
%   invariant polytope and discretized to form the `initial states` for each
%   location. The polytopes of dimension "n-1" are partitioned further in
%   the exact same way as the invariant faces (see below). Currently,
%   polytopes of all other dimensions are not partitioned
%   further. Partitioning procedure for these polytopes are to left for
%   future work. After the partitioning is done, each polytope in the
%   partition is entered as a state under the field ".initstate" in the
%   parent location.
%
%
%
%   Faces of the invariant polytope are partitioned to form `face states` for
%   each location. Each face of the invariant is used as the starting point
%   for the partition. The partitioning scheme depends on the type of the
%   continuous dynamics in each location (see the source code). After the
%   partitioning is done, each polytope in the partition is entered as a
%   state under the parent face and location in "GLOBAL_AUTOMATON".
%
%
%
%   Finally, the reachability analysis is performed for the polytopes
%   associated with all `initial states` using the function "flow_reach",
%   which computes the set of reachable polytopes from the initial
%   poltyope. We use each reachable polytope to refine the partition of the
%   invariant face which that reachable polytope is on further by spliting
%   each poltope in the partition into the part that overlaps with the
%   reachable polytope (the intersection) and the part does not overlap with
%   the reachable polytope (the set difference).
%
% See Also:
%   verify,flow_reach

% input global variable
global GLOBAL_PIHA 

global STOP_VERIFICATION_FLAG

% output global variable
global GLOBAL_AUTOMATON

GLOBAL_AUTOMATON = cell(length(GLOBAL_PIHA.Locations),1);
% GLOBAL_AUTOMATON is a discretization of GLOBAL_PIHA with a few special
% states added to each location

% There are two types of states for each location
%   .initstate     : discretization of initial continuous set for each
%                    location
%   .face{i}.state : discretization of ith invariant face for each location
%
% Each state has the following components
%   .polytope      : Continuous set for this state.
%   .mapping       : Continuous mapping computed from polytope.
%   .children      : Destination states.
%   .null_event    : There exists trajectories that never triggers any
%                    further event from the polytope for this state.
%   .time_limit    : Mapping computation terminated by time limit so the 
%                    the mapping may not contain all states in mapping set.
%   .out_of_bound  : Part of the mapping computed goes out of analysis
%                    region.
%   .indeterminate : Cannot determine further evolution of the system from 
%                    this state. Reasons maybe that polytope for this state 
%                    contains singularities so that mapping cannot be
%                    computed. This flag must be set manually by user.
%   .non_reachable : This state wasn't reached during initial partition 
%							computation. If some state leads to this state it won't 
%							be considered in the reachability analysis, since 
%							it could have happened due to numerical imprecision.
%
%   null_event,time_limit,out_of_bound,indeterminate are boolean flags.
%
%   .terminal      : If there exists a trajectory from the polytope for this
%                    state that causes the FSM to switch to a terminal
%                    FSM state q', i.e the FSM state for which there is no
%                    outgoing transition from any of the component states,
%                    then the cell array .terminal contains q', otherwise it
%                    is empty.

% Initialize initial states for each initial location
%X0 = GLOBAL_PIHA.InitialContinuousSet;
%GLOBAL_AUTOMATON{1}.initstate={};

%Get flag which decides if initial reachability is to be performed or not.

global GLOBAL_APPROX_PARAM
approx_param = GLOBAL_APPROX_PARAM;

init_reach = approx_param.perform_init_reachability;

IC = GLOBAL_PIHA.InitialConditions;
for i=1: length(IC)
    GLOBAL_AUTOMATON{IC{i}.initialLocation}.initstate={};
    X0 =GLOBAL_PIHA.InitialContinuousSet{i};
    init_loc = IC{i}.initialLocation;
    init_cell = IC{i}.initialCells;
    INV = return_init_region(init_cell);   
    
    overall_dynamics = check_overall_dynamics(GLOBAL_PIHA.SCSBlocks, ...
        GLOBAL_PIHA.Locations{init_loc}.q);
    if strcmp(overall_dynamics,'linear')
        [A,b] = overall_system_matrix(GLOBAL_PIHA.SCSBlocks, ...
            GLOBAL_PIHA.Locations{init_loc}.q);
    end
    initstate = {};
    if issubset(X0,INV)
        polytope = X0;
    else
        polytope = X0&INV;
    end;
    if ~isempty(polytope)
        [CE,dE,CI,dI] = linearcon_data(polytope);
        if (length(dE) == 1)
            % if n-1 dimensional patch
            patch = polytope;
            switch overall_dynamics
                
                case 'linear',
                    % First partition the patch into 2 subpatches according to
                    % the direction of the vector field (in/out of invariant). Then
                    % partition further using tolerances.
                    partition = linear_partition(patch,A,b, ...
                        GLOBAL_PIHA.Locations{init_loc}.q);
                    
                case 'nonlinear',
                    partition = nonlinear_partition(patch,GLOBAL_PIHA.SCSBlocks, ...
                        GLOBAL_PIHA.Locations{init_loc}.q);
                    
                case 'clock',
                    % Since there is no variation in vector field, only the size
                    % tolerance is used.
                    partition = clock_partition(patch, ...
                        GLOBAL_PIHA.Locations{init_loc}.q);
                    
                otherwise,
                    error(['Unknown overall system dynamics ''' ...
                            overall_dynamics '''.'])
                        
                end%case
                
            else
                % partition to be coded later for full dimensional X0, do nothing
                % for now
                partition = {polytope};
                
            end%if 
            
            
            for m = [1:length(partition)]
                initstate.polytope = partition{m};
                initstate.mapping = {};
                initstate.children = [];
                initstate.null_event = -1;
                initstate.time_limit = -1;
                initstate.out_of_bound = -1;
                initstate.indeterminate = 0;
                initstate.non_reachable = 0;
                initstate.terminal = {};
                initstate.visited  = 0;
                initstate.cell=init_cell;
                initstate.destination = {};
                GLOBAL_AUTOMATON{init_loc}.initstate{length(GLOBAL_AUTOMATON{init_loc}.initstate)+1} = initstate;
            end%for m
        end%if length
    end %if ~isempty
    
    %There will be an 'initstate' in  GLOBAL_AUTOMATON for every location and Cell location
    %in GLOBAL_AUTOMATON.InitialConditions.
end%for   
% Initialize partition for invariant faces in each location
for k = 1:length(GLOBAL_PIHA.Locations)    
    entry_region ={};
    for m = 1:length(GLOBAL_PIHA.Locations{k}.interior_cells)
        partition=return_region(GLOBAL_PIHA.Locations{k}.interior_cells(m));   
        % >>>>>>>>>>>> Initial Partition -- DJ -- 06/30/03 <<<<<<<<<<<<
        % There are a set of states. One corresponds
        % to interior region and others are boundary states.
        % The for-loop is added by Dong Jia to initialize information for all states.
        for l=1:length(partition)
            entry_region{m}.state{l}.polytope = partition{l};
            entry_region{m}.state{l}.mapping = {};
            entry_region{m}.state{l}.children = [];
            entry_region{m}.state{l}.null_event = -1;
            entry_region{m}.state{l}.time_limit = -1;
            entry_region{m}.state{l}.out_of_bound = -1;
            entry_region{m}.state{l}.indeterminate = 0;
            entry_region{m}.state{l}.non_reachable = 1;
            entry_region{m}.state{l}.terminal = {};
            entry_region{m}.state{l}.visited  = 0;
            entry_region{m}.state{l}.split = 0;
            % >>>>>>>>>>>> Adding Field destination (states) -- DJ -- 06/30/03 <<<<<<<<<<<<
            % The field destination is add to record the reachability results.
            % Added by Dong Jia
            entry_region{m}.state{l}.destination = {};
            % >>>>>>>>>>>> -------------- end (Adding Field destination (states)) --------------- <<<<<<<<<<<<
        end 
        % >>>>>>>>>>>> -------------- end (Initial Partition) --------------- <<<<<<<<<<<<
    end 
    if isempty(entry_region)
        GLOBAL_AUTOMATON{k} = [];
    else
        GLOBAL_AUTOMATON{k}.interior_region = entry_region;
    end 
end 
fprintf(1,'\n')
if init_reach
    %% Refine partition for invariant faces in each location further using
    %% reachability analysis.
    for m = 01:length(GLOBAL_PIHA.InitialConditions)
        % Compute sequence of reachable polytopes for each initial state
        init_loc = GLOBAL_PIHA.InitialConditions{m}.initialLocation;
        init_cell=GLOBAL_PIHA.InitialConditions{m}.initialCells;
        for n=1:length(GLOBAL_AUTOMATON{init_loc}.initstate)
            fprintf(1,'Reachability analysis for loc %d , Cell region %d initial state %d:\n',init_loc,init_cell,n)
            [reachable,time_limit] = ...
                flow_reach(GLOBAL_AUTOMATON{init_loc}.initstate{n}.polytope,init_loc,init_cell);
            % Intersect (subtract) each reachable polytope with (from) all states
            % on the same face.
            % Skip the first state in the reachable set which is always an
            % initial state.
            
            % Changed by Xiyan and Jim K (4/04) to stop if
            % the reachability hit an "Avoid" state.
            assignin('base','reachable',reachable);
            return
            
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.time_limit = time_limit;
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.mapping = reachable(1).mapping;
            % >>>>>>>>>>>> Recording Reachability Results (initial state) -- DJ -- 06/30/03 <<<<<<<<<<<<
            % Recording the reachability results for the initial state.
            % Added by Dong Jia to record complet information from
            % reachability analysis
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.destination=reachable(1).destination;
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.null_event=reachable(1).null_event;
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.time_limit=reachable(1).time_limit;
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.out_of_bound=reachable(1).out_of_bound;
            GLOBAL_AUTOMATON{init_loc}.initstate{n}.terminal=reachable(1).terminal;
            % >>>>>>>>>>>> -------------- end (Recording Reachability Results (initial state)) --------------- <<<<<<<<<<<<
            
            fprintf(1,'\n Finished preliminary reachability analysis. \n');
            % save reachable_data reachable;
            
            prel_refine(reachable);
        end %for n
    end % for m
end %if init_reach

%Section ends here ---------------------------------------------------------------------

return


%------------------------------------------------------------------------------------------

function prel_refine(reachable)


global GLOBAL_AUTOMATON;
global GLOBAL_PIHA;

fprintf(1,'\n Starting preliminary refinement.\n');
len_reachable = length(reachable);

%Loop through the reachable set
for m = 2:length(reachable)
    rlocation = reachable(m).location;
    [dum1,dum2,int_reg] = intersect(reachable(m).cell,GLOBAL_PIHA.Locations{rlocation}.interior_cells);
    polytope = reachable(m).polytope;
    
    %Get current states for this face
    state = GLOBAL_AUTOMATON{rlocation}.interior_region{int_reg}.state;
    refinement = {};
    
    %Loop through current states and break apart based on reachable
    %set mapping.
    for n = 1:length(state)
        temp = state{n};
        overlap = polytope & state{n}.polytope;
        if ~isempty(overlap)
            [CE,dE,CI,dI]=linearcon_data(overlap);
            box = grow_polytope_for_iautopart(CE,dE,CI,dI,rlocation,state{n}.polytope);
            temp.polytope = (box&return_invariant(reachable(m).cell));
            temp.mapping  = reachable(m).mapping;
            temp.non_reachable = 0;
            % >>>>>>>>>>>> Recording Reachability Results (Intermedia States) -- DJ -- 06/30/03 <<<<<<<<<<<<
            % Recording reachability results for intermedia results.
            % Added by Dong Jia to record all information from reachability
            % analysis.
            temp.destination=reachable(m).destination;
            temp.null_event=reachable(m).null_event;
            temp.time_limit=reachable(m).time_limit;
            temp.out_of_bound=reachable(m).out_of_bound;
            temp.terminal=reachable(m).terminal;
            % >>>>>>>>>>>> -------------- end (Recording Reachability Results (Intermedia States)) --------------- <<<<<<<<<<<<
            % >>>>>>>>>>>> Split Flag -- DJ -- 06/30/03 <<<<<<<<<<<<
            % Add a flag split to indicate whether a state is created by splitting a
            % previous state. Added by Dong Jia
            temp.split = 0;
            % >>>>>>>>>>>> -------------- end (Split Flag) --------------- <<<<<<<<<<<<
            refinement{length(refinement)+1} = temp;
            fprintf(1,'\n  state %d/%d created.\n',m,len_reachable);
        end
    end
    
    if length(refinement)>1
        for n=1:length(refinement)
            refinement{n}.split=1;
        end
    end
    
    % This part find new reachable regions and add them to the structure called 'refinement'. 
    % At the end, the structure 'refinement' is copied to 
    % GLOBAL_AUTOMATON{LOCATION}.interior_Region{REGION}.state
    % The cycle then repeats itself for length of structure 'reachable'.
    % >>>>>>>>>>>> Getting States for Current Cell -- DJ -- 06/30/03 <<<<<<<<<<<<
    % Get the state list for current cell to refine.
    % Added by Dong Jia
    curr_states = GLOBAL_AUTOMATON{rlocation}.interior_region{int_reg}.state;
    % >>>>>>>>>>>> -------------- end (Getting States for Current Cell) --------------- <<<<<<<<<<<<
    % >>>>>>>>>>>> State Refinement (while part)-- DJ -- 06/30/03 <<<<<<<<<<<<
    % In the while loop, all states will be checked to see if they intersect
    % with each other. If so, the two intersected states will be partitioned.
    % The following while-end loop is added by Dong Jia
    k=0;
    while 1
        k=k+1;
        if k>length(refinement)
            break;
        end
        % >>>>>>>>>>>> -------------- end (State Refinement (while part)) --------------- <<<<<<<<<<<<
        % >>>>>>>>>>>> Eliminating non_reachable-- DJ -- 06/30/03 <<<<<<<<<<<<
        % The field non_reachable is not used to judge whether one state might need to be
        % refined. The if-end is commented by Dong Jia
        %       if ~(GLOBAL_AUTOMATON{rlocation}.interior_region{int_reg}.state{j}.non_reachable)
        % >>>>>>>>>>>> -------------- end (Eliminating non_reachable) --------------- <<<<<<<<<<<<
        intersect_flag=1;
        for j=1:length(curr_states)
            poly=curr_states{j}.polytope;
            if isfeasible(poly,refinement{k}.polytope)
                inter=refinement{k}.polytope&poly;
                intersect_flag=0;
                % The if-elseif-else-end block 
                % is to consider special cases when one set is a subset of the other one.
                if issubset(refinement{k}.polytope,poly)
                    %   poly contain a state in the refinement list. in
                    %   this case poly will be partitioned. All states
                    %   generated from poly-inter are appended to the
                    %   curr_states list and curr_state{j} is updated by
                    %   inter.
                    GA_not=poly-inter;
                    for i=1:length(GA_not)
                        curr_states{length(curr_states)+1}=curr_states{j};
                        curr_states{length(curr_states)}.polytope=GA_not{i};
                        % >>>>>>>>>>>> Updating split (Case 1) -- DJ -- 06/30/03 <<<<<<<<<<<<
                        % Updating the field split for newly created states
                        % Added by Dong Jia
                        if ~curr_states{j}.non_reachable
                            curr_states{length(curr_states)}.split=1;
                        end
                        % >>>>>>>>>>>> -------------- end (Updating split (Case 1)) --------------- <<<<<<<<<<<<
                    end
                    % >>>>>>>>>>>> Update Information for inter (case 1)-- DJ -- 06/30/03 <<<<<<<<<<<<
                    % Updating reachability information for inter. If curr_state{j} is not a
                    % reachable set computed from  a initial set, update the reachability
                    % infor from refinement{k}. Otherwise, update the reachability infor by the
                    % intersection of the corresponding fields in curr_state{j} and
                    % refinement{k}.
                    % Added by Dong Jia
                    if curr_states{j}.non_reachable
                        curr_states{j}=refinement{k};
                        curr_states{j}.polytope=inter;
                    else
                        curr_states{j}.polytope=inter;
                        if ~isempty(refinement{k}.mapping)
                            if ~isempty(curr_states{j}.mapping)
                                tmp_mapping={};
                                u=1;
                                for s=1:length(curr_states{j}.mapping)
                                    for t=1:length(refinement{k}.mapping)
                                        if isfeasible(refinement{k}.mapping{t},curr_states{j}.mapping{s});
                                            u=u+1;
                                            tmp_mapping{u}=refinement{k}.mapping{t} & curr_states{j}.mapping{s};
                                        end
                                    end
                                end
                                curr_states{j}.mapping = tmp_mapping;
                                tmp_dest={};
                                u=0;
                                for s=1:length(curr_states{j}.destination)
                                    for t=1:length(refinement{k}.destination)
                                        if (refinement{k}.destination{t}.location==curr_states{j}.destination{s}.location) && ...
                                                (refinement{k}.destination{t}.cell==curr_states{j}.destination{s}.cell) && ...
                                                isfeasible(refinement{k}.destination{t}.mapping,curr_states{j}.destination{s}.mapping)
                                            u=u+1;
                                            tmp_dest{u} = curr_states{j}.destination{s};
                                            tmp_dest{u}.mapping= refinement{k}.destination{t}.mapping&curr_states{j}.destination{s}.mapping;
                                            break;
                                        end
                                    end
                                end
                                curr_states{j}.destination=tmp_dest;
                            else
                                curr_states{j}.mapping = refinement{k}.mapping;
                                curr_states{j}.destination=refinement{k}.destination;
                            end
                        end
                        curr_states{j}.null_event=curr_states{j}.null_event & refinement{k}.null_event;
                        curr_states{j}.time_limit=curr_states{j}.time_limit & refinement{k}.time_limit;
                        curr_states{j}.out_of_bound=curr_states{j}.out_of_bound & refinement{k}.out_of_bound;
                        tmp_terminal = {};
                        u=0;
                        for s=1:length(curr_states{j}.terminal)
                            for t=1:length(refinement{k}.terminal)
                                if curr_states{j}.terminal{s}==refinement{k}.terminal{t}
                                    u=u+1;
                                    tmp_terminal{u}=curr_states{j}.terminal{s};
                                    break;
                                end
                            end
                        end
                        curr_states{j}.terminal=tmp_terminal;
                        curr_states{j}.split=refinement{k}.split;
                    end
                    % >>>>>>>>>>>> -------------- end (Update Information for inter (case 1)) --------------- <<<<<<<<<<<<
                    break;
                end
            end %if isfeasible...
            
        end %for j=1....
        if intersect_flag
            curr_states{length(curr_states)+1} = refinement{k};
        end
        %       end %if ~(GLO...
        % >>>>>>>>>>>> -------------- end (State Partition) --------------- <<<<<<<<<<<<
    end %while 1
    GLOBAL_AUTOMATON{rlocation}.interior_region{int_reg}.state = curr_states;
    refinement = {};
end

return

%----------------------------------------------------------------------------

function INV = return_region(cell)

% Get the `invariant` for the given `location`.
%
% Syntax:
%   "INV = return_region(cell)"
%
% Description:
%   Return the "linearcon" object representing the `invariant` polytope for
%   the `location` specified by the "cell".
%
% See Also:
%   piha,linearcon

% The routine is changed by Dong Jia. Instead of return a single linearcon
% object, it returns a set of linearcon objects.

global GLOBAL_PIHA GLOBAL_APPROX_PARAM

% The perform_face_partitioning flag dictates when face partitioning is
% used.  Face partitioning is when a cell region is split up into the
% its center and several sliver regions on the faces of the cell.
% JPK 11/2003
perform_face_partitioning = 0;

boundary = GLOBAL_PIHA.Cells{cell}.boundary;
hpflags = GLOBAL_PIHA.Cells{cell}.hpflags;

C = []; d = [];
for l = 1:length(boundary)
    if hpflags(l)
        C(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    else
        C(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    end
end

if (perform_face_partitioning==0)
    INV{1} = linearcon([],[],C,d);
else
    
    % >>>>>>>>>>>> Patitioning Cell -- DJ -- 06/30/03 <<<<<<<<<<<<
    % Instead creating a state for a cell, currently we create a state for the
    % interior region of the cell and several boundary states.
    % Changed by Dong Jia from
    % INV = linearcon([],[],C,d);
    % to the following codes
    % First, we create a state for the interior region
    INV{1} = linearcon([],[],C,d-GLOBAL_APPROX_PARAM.poly_bloat_tol);
    % Secondly, we create states along each face.
    for i=1:length(d)
        INV{i+1} = clean_up(linearcon([],[],[C;-C(i,:)],[d;-d(i)+GLOBAL_APPROX_PARAM.poly_bloat_tol]));
    end
    % Thirdly, since the states along faces may intersect, we refine those
    % states.
    empty_sets=[];
    i=2;
    while i<length(INV)
        curr_num_inv=length(INV);
        for j=i+1:curr_num_inv
            inter = INV{i} & INV{j};
            if ~isempty(inter)
                if issubset(INV{i},INV{j})
                    INV{i}={};
                    break;
                    empty_sets = [empty_sets; i];
                end
                if issubset(INV{j},INV{i})
                    continue;
                end
                GA1 = INV{i} - inter;
                GA2 = INV{j} - inter;
                INV{i} = GA1{1};
                INV{j} = GA2{1};
                INV{length(INV)+1} = inter;
            end
        end
        i = i + 1;
    end
    
    tmp_INV = {};
    k=0;
    for i=1:length(INV)
        if length(find(empty_sets == i))==0
            k=k+1;
            tmp_INV{k} = INV{i};
        end
    end
    
    INV = tmp_INV;
    % >>>>>>>>>>>> -------------- end (Partitioning Cell) --------------- <<<<<<<<<<<<
end

return

%----------------------------------------------------------------------------

function INV = return_init_region(cell)

% Get the `invariant` for the given `location`.
%
% Syntax:
%   "INV = return_init_region(cell)"
%
% Description:
%   Return the "linearcon" object representing the `invariant` polytope for
%   the `location` specified by the "cell".
%
% See Also:
%   piha,linearcon

% This routine acts the same as the previous return_region routine.

global GLOBAL_PIHA GLOBAL_APPROX_PARAM

boundary = GLOBAL_PIHA.Cells{cell}.boundary;
hpflags = GLOBAL_PIHA.Cells{cell}.hpflags;

C = []; d = [];
for l = 1:length(boundary)
    if hpflags(l)
        C(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    else
        C(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    end
end
INV = linearcon([],[],C,d);

return

%--------------------------------------------------------------------------

function face=get_cell_face(cell,hyperplane)

global GLOBAL_PIHA

%This function takes a GLOBAL_PIHA cell and a hyperplane and returns the polytope that represents
%the face of the cell that is described by the hyperplane.

boundary = GLOBAL_PIHA.Cells{cell}.boundary;
hpflags = GLOBAL_PIHA.Cells{cell}.hpflags;

C = []; d = [];
for l = 1:length(boundary)
    if hpflags(l)
        C(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    else
        C(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.c;
        d(l,:) = -GLOBAL_PIHA.Hyperplanes{boundary(l)}.d;
    end
end

Ci=C;
Di=d;

ce=GLOBAL_PIHA.Hyperplanes{hyperplane}.c;
de=GLOBAL_PIHA.Hyperplanes{hyperplane}.d;
face=linearcon(ce,de,C,d);
face=clean_up(face);

return

%----------------------------------------------------------------------------

function cell=get_terminal_cell(current_location,transitions,face,face_element,hyp)

global GLOBAL_PIHA

%This function finds the cell that represents the terminal location in the 'face_element' of
%the 'transition' array 

cur_cell=GLOBAL_PIHA.Locations{current_location}.p;
neighbors=GLOBAL_PIHA.Cells{cur_cell}.neighbors;
boundary=GLOBAL_PIHA.Cells{cur_cell}.boundary;
hpflags=GLOBAL_PIHA.Cells{cur_cell}.hpflags;
[dum1,dum2,index]=intersect(hyp,boundary);
hpflag=hpflags(index);

%Create a list of regular destination cells to leave out of search for terminal cells.
%Also create index of terminal cells (number is a list of indices to terminal cells).
used=[];
number=[ ];
for j=1:length(transitions{face})
    if strcmp(transitions{face}(j).type,'regular')
        used=[used transitions{face}(j).destination];
    else
        number=[number j];   
    end
end


for k=1:length(used)
    used_cell(k)=GLOBAL_PIHA.Locations{used(k)}.p;
end
%'term_cells' will be a list of cells which are possible terminal cells on this face.
term_cells=setdiff(neighbors,used_cell);

%Compute ALL neighbors that are neighbors on 'this' hyperplane:
next_p=[];
for i=1:length(neighbors)
    neighbors_hp_index=[];
    for j=1:length(GLOBAL_PIHA.Cells{neighbors(i)}.boundary)
        %Flag the neighbors index of boundary hps if it is the same hp as is being tested
        %AND it is on the opposite side of the hp as the current cell
        if GLOBAL_PIHA.Cells{neighbors(i)}.boundary(j)==hyp&GLOBAL_PIHA.Cells{neighbors(i)}.hpflags(j)~=hpflag
            neighbors_hp_index=j;
        end
    end
    if ~isempty(neighbors_hp_index)
        next_p=[next_p neighbors(i)];
    end
end

term_cells_on_face=intersect(term_cells,next_p);


%'term_cells_on_face' will be the legnth of 'number'.  The index into 'number' that corresponds to 'face_element'
%is also the index into 'term_cell' that corresponds to the current terminal cell.
[dum1,dum2,index]=intersect(face_element,number);
cell=term_cells_on_face(index);

return
