function piha(sys)

% Polyhedral invariant hybrid automaton compiler 
%
% This compiler uses a modified method to create a PIHA from the switched
% continuous system.  Instead of dividing the entire state space region with
% every hyperplane associated with all of the PTHB's, this method creates a
% partition of the analysis region for each discrete location.  The analysis 
% region for each location is then partitioned only with the pthb's that
% are relevant for that location (i.e. only the pthb's which define the guards).
%Also, clock information and reset information are collected and attached to the 
%transition information.  
%
%A location is created for each finite state in the
%Stateflow machine(s).  An interior region is defined for each location.  This interior
%region is a collection of pointers to the structure 'cells' which is a collection
%of convex regions (i.e. so the interior regions are made up of a collection of smaller
%convex regions).  
%
%Each location also contains information about each of its 'exiting'
%transitions.  Information about transitions are contained within each 'transition'.
%This information includes:
%
%-'id'	     					the number of the transition (assigned by Stateflow)
%-'expression'					the expression attached to the transition
%-'clock'							the clock number (if there is one)
%-'source'						the source state (identified by the value of 'q')
%-'destination'					the source state (identified by the value of 'q')
%-'destination_name'			the label given to the destination
%-'reset_flag'					set if there is a reset associated with this transition
%-'guard'							pointers to elements in 'cells' that define the guard regions
%-'guard_cell_event_flags'	a matrix where the i'th row is a vector of flags identifying
%									which boundary hyperplane of the i'th guard cell is an event
%									hyperplane
%-'guard_compl'					cells which define the complement of the cells in 'guard'
%
%
%
% Syntax:
%   "HA = piha(sys)"
%
% Description:
%   "piha(sys)" returns a PIHA object.  The returned PIHA object is
%   equivalent to the Simulink CheckMate model "sys" within the analysis
%   region.
%

% "only_condition_inputs_flag" added by JimK (11/2002).  The purpose of the flag
% is to notify "create_guard" if there are only condition inputs into the stateflow
% block.  If this is the case, all edges should be flagged as event edges.

%
% Last change: 11/18/2002 JPK


global GLOBAL_PIHA SSTREE CELLS LOCATIONS

CELLS={};

if ~isstr(sys)
  error(['Input argument must be a string.'])
  return
end

if strcmp(sys,'help')
  fid = fopen('@piha/piha_structure.txt');
  while 1
    line = fgetl(fid);
    if ~isstr(line), break, end
    fprintf(1,'%s\n',line)
  end
  fclose(fid);
  return
end

% Check the model syntax before performing conversion
if ~check_model_syntax(sys)
  fprintf(1,'\007Syntax error, model conversion aborted.\n')
  return
end

% Find Simulink handles for switched continuous system blocks (SCSB),
% finite state machine blocks (FSMB), and polyhedral threshold blocks
% (PTHB) in the CheckMate model.
scsbHandle = find_masked_blocks(sys,'SwitchedContinuousSystem');
fsmbHandle = find_masked_blocks(sys,'Stateflow');
pthbHandle = find_masked_blocks(sys,'PolyhedralThreshold');
clockHandle = find_masked_blocks(sys,'VariableZeroOrderHold');

clockList=[];
%Create a list of all clocks and their associated parameters
for c=1:length(clockHandle)
   clockList{c}.name  = get_param(clockHandle(c),'name');
   clockList{c}.period =evalin('base',get_param(clockHandle(c),'period'));
   clockList{c}.jitter =evalin('base',get_param(clockHandle(c),'jitter'));
   clockList{c}.phase =evalin('base',get_param(clockHandle(c),'theta'));
end

% **********************************
% (1) Construct list of hyperplanes.
% **********************************

% Get threshold hyperplanes from the PTHBs in the simulink model.
disp('Compiling threshold hyperplanes.')
NBDHP = {};
for k = 1:length(pthbHandle)
    [Ck,dk] = augment_poly_constraints(scsbHandle,pthbHandle(k));
    pthb{k}.c=Ck;
    pthb{k}.d=dk;
    pthb{k}.hps=[];
    for l = 1:length(dk)  
        new = length(NBDHP)+1;
        NBDHP{new}.pthb = k;
        NBDHP{new}.index = l;
        NBDHP{new}.c = Ck(l,:);
        NBDHP{new}.d = dk(l,:);
    end%for
end%for

%Create hyperplane list
AR = get_analysis_region(scsbHandle);
[pthb,NAR]=create_hyperplanes(NBDHP,AR,pthb);


% **************************************************************************
% (3) Build data base for Stateflow blocks that will be used for conversion.
% **************************************************************************

machine_id = get_machine_id(sys);
if isempty(machine_id)
  fprintf(1,['\007Error: Cannot find machine id for ''' sys '''!!!\n'])
  return
end

sfdata = {};

for k = 1:length(fsmbHandle)

    block_name = get_param(fsmbHandle(k),'Name');
    chart_id = find_chart_id(machine_id,block_name);
  
    if isempty(chart_id)
        fprintf(1,['\007Error: Cannot find chart id for "' block_name '"!!!\n'])
        return
    else
    
        data_id = sf('find',sf('DataOf',chart_id),'.scope','INPUT_DATA');
        input_data = {};
        clock=[];
        in_num=0;

        for l = 1:length(data_id)
      
            [expression,clock_handle]=sf_input_expression(fsmbHandle(k),'data',l);
            if ~isempty(clock_handle)
                error(['Clocks must be event inputs.'])
            else
                in_num=in_num+1;
                input_data{in_num}.Name = sf('get',data_id(l),'.name');
                if input_data{in_num}.Name(1)==' '|input_data{in_num}.Name(length(input_data{in_num}.Name))==' '
                            error('The names of Stateflow input data variables cannot have leading or trailing spaces.')
                end
                        
                input_data{in_num}.Expression = expression;
      	    end%if
        
        end%for
    
        event_id = sf('find',sf('EventsOf',chart_id),'.scope','INPUT_EVENT');
        input_event = {}; 

        for l = 1:length(event_id)

            event_name = sf('get',event_id(l),'.name');
            if event_name(1)==' '|event_name(length(event_name))==' '
                            error('The names of Stateflow input events cannot have leading or trailing spaces.')
            end
            if ~strcmp(event_name,'start')

                [expression,clock_handle]=sf_input_expression(fsmbHandle(k),'event',l);

                if ~isempty(clock_handle)
                    clock_num=length(clock)+1;
                    clock{clock_num}.Name=sf('get',event_id(l),'.name');
                    clock{clock_num}.ClockBlock=get_param(clock_handle,'name');
                    clock{clock_num}.ClockHandle=clock_handle;
                else

       
               	    % ignore the start event, which is used only for simulation purpose
        	        new = length(input_event)+1;
        	        input_event{new}.Name = event_name;
    
                    if ~isempty(sf('find',event_id(l),'.trigger','RISING_EDGE_EVENT'))
                	    input_event{new}.Trigger = 'RISING_EDGE_EVENT';   
        	         end

                    if ~isempty(sf('find',event_id(l),'.trigger','FALLING_EDGE_EVENT'))
    	                input_event{new}.Trigger = 'FALLING_EDGE_EVENT';   
    	            end

                    if ~isempty(sf('find',event_id(l),'.trigger','EITHER_EDGE_EVENT'))
                	    input_event{new}.Trigger = 'EITHER_EDGE_EVENT';   
                    end
  
                     if strcmp('RISING_EDGE_EVENT',input_event{new}.Trigger)
                        input_event{new}.Expression = ...
                     	 sf_input_expression(fsmbHandle(k),'event',l);
                    elseif strcmp('FALLING_EDGE_EVENT',input_event{new}.Trigger)
                         input_event{new}.Expression = ...
                 	    ['~(' sf_input_expression(fsmbHandle(k),'event',l) ')'];
                    else
                        error('This version only supports  rising and falling event edges!')
                    end%if
               
                end%if
            end%if
        end%for
    
        sfdata{k}.BlockName = block_name;
        sfdata{k}.SimulinkHandle = fsmbHandle(k);
        sfdata{k}.StateflowChartID = chart_id;
        sfdata{k}.InputData = input_data;
        sfdata{k}.InputEvent = input_event;
        sfdata{k}.Clock = clock;
           
    end%if
    
end%for

% **************************************************************
% (4) Find initial discrete states for all the Stateflow blocks.
% **************************************************************

% Get initial state id for each Stateflow block, assuming that there is 
% only one default transition per machine.
q0 = []; 
for k = 1:length(sfdata)
  chart_id = sfdata{k}.StateflowChartID;
  DTid = sf('DefaultTransitionsOf',chart_id);
  q0id = sf('get',DTid,'.dst.id');
  q0(k) = q0id;
end

%**************************************************************
%Create list of locations and all guards for each location
%**************************************************************
k=ones(1,length(sfdata));
%Find the total number of possible locations
no_loc=1;
for j=1:length(sfdata)
   machine{j}.states=sf('get',sfdata{j}.StateflowChartID,'.states');
   no_loc=no_loc*length(machine{j}.states);
end
locations=[];

% Reset operation. It maps the FSMB to SCSB
% Ansgar Fehnker 09/16/2002
 

% Compute which fsmb is connceted to which scsb
for k0 =1:length(fsmbHandle)
  fsmb2scsb{k0} = [];
end

for k1 = 1:length(scsbHandle)
  if strcmp(get_param(scsbHandle(k1),'use_reset'),'on')
    inputfsmb_Handle = trace_scsb_input(scsbHandle(k1),'reset');
    % The error checking in trace_scsb_input ensures that only one FSMB
    % is returned in inputfsmb_Handle.
    fsmbindex = find(inputfsmb_Handle == fsmbHandle);
    fsmb2scsb{fsmbindex} = [fsmb2scsb{fsmbindex} k1];
  end
end
    
%Find all possible locations by attempting all combinations of states
%and machines
for j=1:no_loc
   
   %Test each combination to see if it is a null state
   for i=1:length(sfdata)
      q(i)=machine{i}.states(k(i));
   end
   null_state=is_terminal_state(q);
   if null_state==0
      new=length(locations)+1;
      number_trans=0;
      for l=1:length(q)
         % "only_condition_inputs_flag" added by JimK (11/2002).  The purpose of the flag
         % is to notify "create_guard" if there are only condition inputs into the stateflow
         % block.  If this is the case, all edges should be flagged as event edges.
         if isempty(sfdata{l}.InputEvent)
            only_condition_inputs_flag = 1;
         else
            only_condition_inputs_flag = 0;
         end
      	transitions=sf('get',q(l),'.srcTransitions');
      	names=sf('get',transitions,'.labelString');
         for p=1:size(names,1)
            number_trans=number_trans+1;
            locations{new}.transitions{number_trans}.id=transitions(p);
            [event_expr,condition_expr,clock,reset_flag]=process_label_string(names(p,:),sfdata,l,clockHandle);
            [total_expression,event_expression,condition_expression]=find_cond_expr(sfdata,condition_expr,event_expr,l);            
            locations{new}.transitions{number_trans}.expression=total_expression;
            locations{new}.transitions{number_trans}.clock=clock;
                        locations{new}.transitions{number_trans}.idx=l;    % Zhi add this to record the index of transition in q
            locations{new}.transitions{number_trans}.source=sf('get',transitions(p),'.src.id');
            locations{new}.transitions{number_trans}.destination=sf('get',transitions(p),'.dst.id');
            locations{new}.transitions{number_trans}.destination_name=sf('get',sf('get',transitions(p),'.dst.id'),'.name');
            locations{new}.transitions{number_trans}.reset_flag=reset_flag;

            % Stores the scsb that defines the reset
            locations{new}.transitions{number_trans}.reset_scs_index=fsmb2scsb{l};

            %Find guard region.  Result will be pointers to cell locations which describe this guard
            [guard_cells,guard_cell_event_flags,guard_compl_cells]=create_guard(condition_expression,event_expression,pthbHandle,pthb,NAR,only_condition_inputs_flag);
            locations{new}.transitions{number_trans}.guard=guard_cells;
            locations{new}.transitions{number_trans}.guard_cell_event_flags=guard_cell_event_flags;
            locations{new}.transitions{number_trans}.guard_compl=guard_compl_cells;

         end %for
      end%for
      %Use guard complements from every transition from this location to compute
      %the 'invariant' for this location.  The idea is to intersect all of the
      %complements of the guards for this location.  This is done iteratively by
      %intersecting the first guard complement with the next, then taking the result 
      %and intersecting it with the 3rd and so on (i.e  ( A and B ) and C ....)
%       clean_loc=clean_transition(locations{new});
%        locations{new}.transitions=clean_loc.transitions;
           locations{new}.q=q;
      locations{new}.state=sf('get',q,'.name');
      locations{new}.interior_cells=[];
           invariant=intersect_complements(locations,new);
      locations{new}.interior_cells=invariant;
   end%if
   
      
   
   for n=1:length(k)
      if k(n)+1<=length(machine{n}.states)
            k(n)=k(n)+1;
            break;
      else
            k(n)=1;
      end%if
   end%for
end%for


% **********************************************************************
% (4.1) Reposition the locations to make the initial locations the first ones.
% **********************************************************************


if (size(q0,1)~=1) | ~all(locations{1}.q==q0(1,:))   
    relocation_count = 1;
    for i=1:size(q0,1)
        for j=1:length(locations)
            if all(locations{j}.q==q0(i,:))

                location_to_move =locations{j};
                location_from_move=locations{relocation_count};
                locations{relocation_count}= location_to_move;
                locations{j}= location_from_move;
                relocation_count=relocation_count +1;
                
            end%if
            if relocation_count > size(q0,1)
                break
            end
        end%for
    end%for
end%end

% **********************************************************************
% (5) Create lists of blocks for each block type in the CheckMate model.
% **********************************************************************

% (i) Switched continuous system blocks (SCSB)
scsbList = cell(length(scsbHandle),1);
use_sd=0;
for k = 1:length(scsbHandle)
  scsbList{k}.name = get_param(scsbHandle(k),'Name');
  scsbList{k}.nx = eval(get_param(scsbHandle(k),'nx'));
  scsbList{k}.nz = eval(get_param(scsbHandle(k),'nz'));
  scsbList{k}.nup = eval(get_param(scsbHandle(k),'nup'));
  if strcmp(get_param(scsbHandle(k),'use_sd'),'on')
      use_sd=1;
  end
  scsbList{k}.nu = eval(get_param(scsbHandle(k),'nu'));
  scsbList{k}.swfunc = get_param(scsbHandle(k),'swfunc');
  % parametric information
  scsbList{k}.pacs = evalin('base',get_param(scsbHandle(k),'PaCs'));
  scsbList{k}.paradim = length(evalin('base',get_param(scsbHandle(k),'p0')));
  inputfsmb_Handle = trace_scsb_input(scsbHandle(k),'input');

  fsmbindices = [];
  for l = 1:length(inputfsmb_Handle)
    fsmbindices(l) = find(inputfsmb_Handle(l) == fsmbHandle);
  end
  scsbList{k}.fsmbindices = fsmbindices;
end

% (ii) Finite state machine blocks (FSMB)
fsmbList = cell(length(sfdata),1);
for k = 1:length(sfdata)
  chart_id = sfdata{k}.StateflowChartID;
  state_id = sf('find','all','state.chart',chart_id);
  states = {};
  for l = 1:length(state_id)
    state_number = get_state_number(state_id(l));
    states{state_number} = sf('get',state_id(l),'.name');
  end
  fsmbList{k}.name = get_param(sfdata{k}.SimulinkHandle,'Name');
  fsmbList{k}.states = states;
end

% (iii) Polyhedral threshold blocks (PTHB)
pthbList = cell(length(pthbHandle),1);
for k = 1:length(pthbHandle)
  pthbList{k}.name = get_param(pthbHandle(k),'Name');
end

% Finally, replace all Stateflow state id by the enumeration
% given in the label (the entry action) of each state.
for k = 1:length(locations)
  locations{k}.q = get_labeled_state(locations{k}.q);
  for l = 1:length(locations{k}.transitions)
    	locations{k}.transitions{l}.destination = ...
          get_labeled_state(locations{k}.transitions{l}.destination);
       locations{k}.transitions{l}.source=...
          get_labeled_state(locations{k}.transitions{l}.source);

  end
end

%*************************************************************************************
%Find inital regions and initial states
%*************************************************************************************

% Get initial continuous set from the parameter block
X0 = get_initial_continuous_set(scsbHandle);

% Identify initial cells from intersection with initial continuous set X0.
% P0 contains all indices (to SSTREE) for the leaf nodes corresponding
% to initial cells.
P0 = [];
for k = 1:length(X0)
      test=find_initial_cells(X0{k},locations{1});
      if ~isempty(test)
         P0 = union(P0,test);
      end
end

L0=[ones(length(P0),1) P0'];
for location_counter=1:length(locations)
    locations{location_counter}.orig_interior_cells=locations{location_counter}.interior_cells;
end

[initial_conditions , locations] = find_initial_conditions(X0,locations);

% Conversion completed, return the PIHA object.
% GLOBAL_PIHA.Hyperplanes should already be assigned.  Assign 
% everything else here.
GLOBAL_PIHA.NAR                  = NAR;
GLOBAL_PIHA.InitialContinuousSet = X0; 
GLOBAL_PIHA.InitialDiscreteSet   = {get_labeled_state(q0)};
GLOBAL_PIHA.Cells                = CELLS;
GLOBAL_PIHA.InitialCells         = P0;
GLOBAL_PIHA.Locations            = locations;
GLOBAL_PIHA.InitialConditions    = initial_conditions;
GLOBAL_PIHA.InitialLocation_Cells= L0;
GLOBAL_PIHA.CLOCKBlocks				= clockList;
GLOBAL_PIHA.SCSBlocks            = scsbList;
GLOBAL_PIHA.PTHBlocks            = pthbList;
GLOBAL_PIHA.FSMBlocks            = fsmbList;
GLOBAL_PIHA.use_sd               = use_sd; 
GLOBAL_PIHA.sample_time          = str2num(get_param(sys,'FixedStep'));

disp('PIHA conversion successful.')
return

%------------------------------------------------------------------------------

function [pthb,NAR]=create_hyperplanes(NBDHP,AR,pthb)

global GLOBAL_PIHA
global GLOBAL_APPROX_PARAM
global GLOBAL_OPTIM_PAR

hyperplanes = [];

% This function creates the list of hyperplanes. It originally resided in the 
% 'partition_ss' function, but it has become necessary to have the hp's before
% the state space is divided. 

% Put hyperplanes constituting the boundary of AR at the beginning of the
% hyperplane list.

[CE,dE,CAR,dAR] = linearcon_data(AR);
hyperplanes = {};
for k = 1:length(dAR)
  hyperplanes{k}.pthb = -1;
  hyperplanes{k}.index = k;
  hyperplanes{k}.c = CAR(k,:);
  hyperplanes{k}.d = dAR(k);
end
NAR = length(hyperplanes);

hyperplane_tol = GLOBAL_APPROX_PARAM.poly_hyperplane_tol;
counter_hyperplanes=NAR;
for k = 1:length(NBDHP)
    dup=0;
  for j=1:counter_hyperplanes
        c1 = hyperplanes{j}.c;
        b1 = hyperplanes{j}.d;
        c =NBDHP{k}.c;
        b=NBDHP{k}.d;
        MATRIX = [c1 b1; c b];
        if rank(MATRIX,hyperplane_tol) < 2 
         dup=1;
         i=NBDHP{k}.pthb;
         pthb{i}.hps=[pthb{i}.hps j];
         break;
     else
         dup=0;
     end
  end
  if dup==0
  counter_hyperplanes = counter_hyperplanes+1;
  hyperplanes{counter_hyperplanes}.pthb = NBDHP{k}.pthb;
  hyperplanes{counter_hyperplanes}.index = NBDHP{k}.index;
  hyperplanes{counter_hyperplanes}.c = NBDHP{k}.c;
  hyperplanes{counter_hyperplanes}.d = NBDHP{k}.d;
  end
end

%Sort out which HP's go with which pthb
for n=NAR+1:length(hyperplanes)
   j=hyperplanes{n}.pthb;
   if j~=-1
      pthb{j}.hps=[pthb{j}.hps n];
   end
end

GLOBAL_PIHA.Hyperplanes = hyperplanes;

return

%------------------------------------------------------------------------------

function [guard_cells,guard_cell_event_flags,guard_compl_cells] = ...
    create_guard(cond_expr,event_expr,pthbHandle,pthb,NAR,only_condition_inputs_flag)

% This procedure divides up the analysis region and returns pointers
% to cells that describe a guard region.

global CELLS 

guard_cells=[];
guard_compl_cells=[];
guard_cell_event_flags=[];

% Find all PTHBs which appear in condition expression.
guard_pthbs = [];
for i = 1:length(pthbHandle)
  name = get_param(pthbHandle(i),'name');
  % Need to fix the code below since findstr() will return all
  % matching substrings, which may be incorrect.
  if ~isempty(findstr(cond_expr,name))
    guard_pthbs=[guard_pthbs i];
  end   
end

% Find all PTHBs which appear in event expression. Also, create the
% list of all hyperplanes associated with event PTHBs.
event_pthbs = [];
event_hps = [];
for i = 1:length(pthbHandle)
  name = get_param(pthbHandle(i),'name');
  % Need to fix the code below since findstr() will return all
  % matching substrings, which may be incorrect.
  if ~isempty(findstr(event_expr,name))
    event_pthbs = [event_pthbs i];
    event_hps = union(event_hps,pthb{i}.hps);
  end
end

% Find overall transition expression, which is the conjunction of
% event and condition expressions.
if ~isempty(cond_expr) & ~isempty(event_expr)
  trans_expr= ['(' event_expr ')&(' cond_expr ')'];
elseif ~isempty(event_expr)
  trans_expr = event_expr;  
elseif ~isempty(cond_expr)
  trans_expr = cond_expr;
else
  error('No transition expression found.')
end

% Initialize the state-space partition with the analysis region as
% a single cell in the partition. The partition will be refined
% iteratively by subtracting each PTHB off each cell in the
% partition in each pass.
[CAR,dAR] = cell_ineq([1:NAR],ones(1,NAR));
partition.poly = linearcon([],[],CAR,dAR);
partition.pthflags = -1*ones(1,length(pthbHandle));

% Refine the state space partition with PTHBs associated with
% condition and event expressions, respectively.
pthbs_used = [guard_pthbs event_pthbs];
partition = partition_ss(partition,pthbs_used,pthb);

% Classify each cell in the partition created above as 'guard' or
% 'guard complement' cells. Add new cells in the partition just
% created to the global cell list CELLS. The result of this function
% will point to regions in this cell list.
for k = 1:length(partition)
  cells_length = length(CELLS);
  
  % Find the boundary hyperplane indices along with their direction
  % and event flags in the global hyperplane list HYPERPLANES.
  [CEk,dEk,CIk,dIk] = linearcon_data(partition(k).poly);
  [boundary,hpflags] = ineq2cell(CIk,dIk);
  pthflags = partition(k).pthflags;
  
  % Test to see if the current cell is a 'repeat' of a region that was
  % already created in the global cell list CELLS.
  repeat_test = is_repeat(boundary,hpflags);
  if repeat_test == 0 
    new = length(CELLS)+1;
    CELLS{new}.boundary = boundary;
    CELLS{new}.hpflags = hpflags;
    CELLS{new}.pthflags = pthflags;
  end
  
  % Test to see if current cell satisfies overall transition
  % expression. If it does, add current cell to guard list. If it does
  % not, add current cell to guard complement list.
  
  % First replace PTHB names in the transition expression with their
  % logical values for the current cell.
  test_expr = trans_expr;
  for pthb_index = pthbs_used
    flag = pthflags(pthb_index);
    name = get_param(pthbHandle(pthb_index),'name');
    % Need to fix the code below since strrep() will also replace
    % matching substrings, which may be incorrect.
    test_expr = strrep(test_expr,name,num2str(flag));
  end
  % Now evaluate transition expression.
  test = eval(test_expr);
  
  if test
    % Compute event flags for the current cell boundary hyperplanes.
    if only_condition_inputs_flag
      event_flags = ones(size(boundary));
    else
      event_flags = zeros(size(boundary));
    end
    [temp,iboundary,ievent_hps] = intersect(boundary,event_hps);
    event_flags(iboundary) = 1;
    guard_cell_event_flags{end+1} = event_flags;
    % Add the current cell index (w.r.t. CELLS) to the list of guard cells.
    if repeat_test == 0
      guard_cells = [guard_cells new];
    else
      guard_cells = [guard_cells repeat_test];
    end
  else
    % Add the current cell index (w.r.t. CELLS) to the list of guard
    % complement cells.
    if repeat_test == 0
      guard_compl_cells = [guard_compl_cells new];
    else
      guard_compl_cells = [guard_compl_cells repeat_test];
    end
  end
  
end

return
% -----------------------------------------------------------------------------

function result = is_terminal_state(q)

result = 1;
for k = 1:length(q)
  parent_chart = sf('get',q(k),'.chart');
  transitionsAll = sf('get',parent_chart,'.transitions');
  qk_transitions = sf('find',transitionsAll,'.src.id',q(k));
  if ~isempty(qk_transitions)
    result = 0;
    break;
  end
end
return

%------------------------------------------------------------------------------
function [total_expression,event_expression,condition_expression] = find_cond_expr(sfdata,cond_expr,event_expr,sf_machine)

%This function will take the condition expression and represent it as
%a logical function in terms of the PTHB's

data=sfdata{sf_machine}.InputData;
condition_expression=cond_expr;
if ~isempty(cond_expr)
    if cond_expr(1)==' '|cond_expr(length(cond_expr))==' '
        error('Stateflow condition xpressions cannot have leading or trailing spaces.')    
    end
end
total_expression=cond_expr;

ok=0;
for i=1:length(data)
   name=data{i}.Name;
   where=findstr(name,cond_expr);
   if (~isempty(where))&length(cond_expr)>=length(name)
   replace_at=[];
for n=where
      
      %The following code only checks to see if the character before and after the 
      %matched strings are non-characters.  This protects against the following case:
      %cond_expr='region1==1'  and sfdata.name='region'
      %******************************************************************************
   if n>1&(n+length(name))<=length(cond_expr)
      if ~(isletter(cond_expr(n-1))|isletter(cond_expr(n+length(name)))|...
               ~isempty(findstr(cond_expr(n-1),'0123456789'))|...
               ~isempty(findstr(cond_expr(n+length(name)),'0123456789')))
         replace_at=[replace_at n];
      end
   elseif (n+length(name))<=length(cond_expr)
		   if ~(isletter(cond_expr(n+length(name)))|~isempty(findstr(cond_expr(n+length(name)),'0123456789')))
            replace_at=[replace_at n];
         end
   elseif n>1
      if ~(isletter(cond_expr(n-1))|~isempty(findstr(cond_expr(n-1),'0123456789')))
         replace_at=[replace_at n];
      end
      
   end %end if where(...
		%******************************************************************************   
        
end %end for n=where

    %Perform the all replacements necessary for the 'name' condition
    index_old=1;
    condition_expression_new=[];
    for j=1:length(replace_at)
            condition_expression_new = [condition_expression_new condition_expression(index_old:replace_at(j)-1)];                
            condition_expression_new = [condition_expression_new data{i}.Expression];
            index_old=replace_at(j)+length(name);
    end
    condition_expression_new = [condition_expression_new condition_expression(index_old:length(condition_expression))];
    condition_expression = condition_expression_new;
    cond_expr=condition_expression;

end %end if (~isempty...
   
  end %end for i=1:...
  
  event_expression=[];
  %Now concatenate the event expression onto the condition expression:
  for i=1:length(sfdata{sf_machine}.InputEvent)
     input_expression=sfdata{sf_machine}.InputEvent{i}.Expression;
     input_event_name=sfdata{sf_machine}.InputEvent{i}.Name;
     if strcmp(deblank(input_event_name),deblank(event_expr))
        if ~isempty(condition_expression)
        	total_expression=strcat('(',input_expression,')&(',condition_expression,')');
        else
         total_expression=input_expression;
        end   
        event_expression=input_expression;
         break;
     end
     
  end
  



return

% -----------------------------------------------------------------------------

function [event_expr,cond_expr,clock,reset_flag] = process_label_string(labelString,sfdata,sf_machine,clockHandle)

% Assume that the condition expression lies between brackets
% (this should have been checked previously).
%Also assume that if there is a clock associated with this transition, it
%appears as the first element in the expression and that it is 'and'ed 
%with the rest of the expression.  
%Example: [(clock1==1)&((A==1)&B==1))]
clock=[];
no_condition=0;
start = findstr(labelString,'[');

%Find where the reset event starts (if there is one)
reset_start=findstr(labelString,'/');
reset_flag=0;

if ~isempty(start)
   event_expr = labelString(1:start-1);
elseif ~isempty(reset_start)
   event_expr = labelString(1:reset_start-1);
else 
   event_expr= labelString;   
end

% Test to see if the SF machine has event inputs but this transition
% does not.  Added by JPK (11/2002).
if isempty(event_expr)&~isempty(sfdata{sf_machine}.InputEvent)
   error(['Stateflow machine has event inputs, but at least one transition '...
   'does not have an event associated with it.']);
end

if ~isempty(reset_start)
if ~isempty(findstr('reset',labelString(reset_start+1:length(labelString))))
         reset_flag=1;
else
         error('Invalid reset event.')
end
end


  stop = findstr(labelString,']');
  is_clock=0;
  
  %Test for a clocked event in the event section of the labelstring
  for i=1:length(sfdata{sf_machine}.Clock)
     name=sfdata{sf_machine}.Clock{i}.Name;
     name=deblank(name);
     is_clock=strcmp(name,event_expr);
     if is_clock
        event_expr=[];
        zoh_block=sfdata{sf_machine}.Clock{i}.ClockHandle;           
        [dum1,dum2,clock]=intersect(zoh_block,clockHandle);
        break;
     end
  end
 
 cond_expr = labelString(start+1:stop-1);

 
  
return

% ----------------------------------------------------------------------------

function machine_id = get_machine_id(sys)

% the Stateflow function returns all block names whose begining substring
% matches with the name given in sys. thus, we have to make sure that we get
% an exact match.

possible_id = sf('find','all','machine.name',sys);
machine_id = [];
for k = 1:length(possible_id)
  name = sf('get',possible_id(k),'.name');
  if strcmp(name,sys)
    machine_id = possible_id(k);
    break;
  end
end
return

% ----------------------------------------------------------------------------

function chart_id = find_chart_id(machine_id,block_name)

possible_id = sf('find','all','chart.machine',machine_id, ...
    'chart.name',block_name);
chart_id = [];
for l = 1:length(possible_id)
  name = sf('get',possible_id(l),'.name');
  if strcmp(name,block_name)
    chart_id = possible_id(l);
    break;
  end
end
return

% ----------------------------------------------------------------------------

function X0 = get_initial_continuous_set(scsbH)

% Get initial continuous set (ICS) for each SCS block. Each ICS is a cell
% array of @linearcon objects.
ICS = {};
for k = 1:length(scsbH)
  ICS{k} = evalin('base',get_param(scsbH(k),'ICS'));
  if (length(ICS{k}) < 1)
    blockname = get_param(scsbH(k),'name');
    error(['Invalid initial continuous set specified for block ''' ...
	blockname '''.'])
  end
end

X0 = {};
idx = ones(1,length(scsbH));
stop = 0;
while ~stop
  % Compose combinations of overall ICS from ICS for each SCS block.
  ICS_CE = []; ICS_dE = [];
  ICS_CI = []; ICS_dI = [];
  for k = 1:length(scsbH)
    [CEk,dEk,CIk,dIk] = linearcon_data(ICS{k}{idx(k)});
    ICS_CE = [           ICS_CE            zeros(size(ICS_CE,1),size(CEk,2))
      zeros(size(CEk,1),size(ICS_CE,2))           CEk               ];
    ICS_dE = [ICS_dE; dEk];		 
    ICS_CI = [           ICS_CI            zeros(size(ICS_CI,1),size(CIk,2))
      zeros(size(CIk,1),size(ICS_CI,2))           CIk               ];
    ICS_dI = [ICS_dI; dIk];
  end
  % Put each combination into the cell array
  X0{length(X0)+1} = linearcon(ICS_CE,ICS_dE,ICS_CI,ICS_dI);

  % Increment the combination index
  k = length(scsbH);
  while (k >= 1)
    idx(k) = idx(k) + 1;
    if (idx(k) > length(ICS{k}))
      idx(k) = 1;
      k = k - 1;
    else
      k = -1;
    end
  end
  stop = (k == 0);
end

return

% ----------------------------------------------------------------------------

function AR = get_analysis_region(scsbH)

% Compose overall analysis region from analysis region for each SCS block.
CAR = [];
dAR = [];
for k = 1:length(scsbH)
  ARk = evalin('base',get_param(scsbH(k),'AR'));
  [CE,dE,Ck,dk] = linearcon_data(ARk);
  if ~isempty(CE) | ~isempty(dE)
    blockname = get_param(scsbH(k),'name');
    error(['Invalid analysis region for block ''' ...
	  blockname '''.'])
  end
  CAR = [          CAR                zeros(size(CAR,1),size(Ck,2))
    zeros(size(Ck,1),size(CAR,2))           Ck                ];
  dAR = [dAR; dk];		 
end
AR = linearcon([],[],CAR,dAR);
return

% ----------------------------------------------------------------------------

function state_number = get_state_number(state_id)

% We assume the label strings in the Stateflow diagram have the following 
% format.
%
%      <state_name>
%      entry: u = <state_number>;
%

labelString = sf('get',state_id,'.labelString');
% Skip all the white spaces at the end.
k = length(labelString);
while (k >= 1) & (isspace(labelString(k)) | (labelString(k) == ';'))
  k = k - 1;
end
% Search backwards until a non-numeric character is found.
labelString = labelString(1:k);
while (k >= 1) & (labelString(k) >= '0') & (labelString(k) <= '9')
  k = k - 1;
end
labelString = labelString(k+1:length(labelString));
state_number = str2num(labelString);
return

% ----------------------------------------------------------------------------

function state_number = get_labeled_state(state_id)

state_number = [];
for k = 1:length(state_id)
  state_number(k) = get_state_number(state_id(k));
end
return

% ---------------------------------------------------------------------------

function icell = find_initial_cells(X0,locations)

%This function intersects linearcon object representing one piece of ICS with
%each CELL in invariant location

global CELLS

icell=[];
initial_invariant=locations.interior_cells;


for i=1:length(initial_invariant)
	[C,d] = cell_ineq(CELLS{initial_invariant(i)}.boundary,CELLS{initial_invariant(i)}.hpflags);
	INV = linearcon([],[],C,d);
	if isfeasible(X0,INV)
   	icell=[icell initial_invariant(i)];
	end
end

%Test to see if any of the initial continuous set either a.) satifies a guard or b.) is outside of the 
%analysis region.
transitions=locations.transitions;
guards=[];
for k=1:length(transitions)
   guards=union(guards,transitions{k}.guard);
end

for i=1:length(guards)
	[C,d] = cell_ineq(CELLS{guards(i)}.boundary,CELLS{guards(i)}.hpflags);
	GUARD = linearcon([],[],C,d);
	if isfeasible(X0,GUARD)
      warning(['Some section of the initial continuous set satisfies the guard of the initial location.'...
      	'  This section will be neglected.'])
	end
end





return

% ----------------------------------------------------------------------------
function [initial_conditions,locations_updated] = find_initial_conditions(X0,locations_list)

%This function finds the list of continuous set, the discrete set, the initial cells
% and the initial location for the initial conditions set X0.
% INPUT
% X0 : a Linearcon object with the state-space region defined as the initial condition
% locations_list : set of locations. We will assume that the default initial location is one (01)
% scsbList : List of SCSB with parameters. For reset purposes.
%
% OUTPUT
% initial_conditions: Cell array with the following fields:
%
%   .continuousSet    = linearcon object with the region for the initial region
%   .initialCells          = cell where the initial conditon is located
%   .discreteCells      = configuration of the SFM when in this element (value of vector q)
%   .initialLocation    = Location where the initial condition is located
%
% locations: list of locations updated for some guard;
% In the future, this routine will receive a structure containing the state (q) and X0.

global CELLS

ICL{1}.location = 01;
ICL{1}.X0        = X0;

for counter_list = 1:length(ICL)

    init_loc = ICL{counter_list}.location;
    X0       = ICL{counter_list}.X0;
    initial_invariant=locations_list{init_loc}.interior_cells;
    init_cond ={};
    init_counter = 1;

    for j=1:length(X0)

        for i=1:length(initial_invariant)

            [C,d] = cell_ineq(CELLS{initial_invariant(i)}.boundary,CELLS{initial_invariant(i)}.hpflags);
    	    INV = linearcon([],[],C,d);

            if isfeasible(X0{j},INV)

                init_counter = length(init_cond);
                init_cond{init_counter+1}.continuousSet      = and(X0{j},INV);
                init_cond{init_counter+1}.initialCells            = initial_invariant(i);
                init_cond{init_counter+1}.discreteSet           = locations_list{01}.q;
                init_cond{init_counter+1}.initialLocation      = 01; 
                
           end%if
           
       end%if
       
    end%if

    %Test to see if any of the initial continuous set either a.) satifies a guard or b.) is outside of the 
    %analysis region.
    disc_state = locations_list{init_loc}.q;
    transitions=locations_list{init_loc}.transitions;

    for i=1:length(transitions)
   
        guards=transitions{i}.guard;

        for j=1:length(guards)
    	    [C,d] = cell_ineq(CELLS{guards(j)}.boundary,CELLS{guards(j)}.hpflags);
    	    GUARD = linearcon([],[],C,d);

            for m=1:length(X0)

                if isfeasible(X0{m},GUARD)

                    if isempty(transitions{i}.clock)

                        warning(['Some section of the initial continuous set satisfies the guard of the initial location.'...
                      	'  This section will be neglected.']);
                
                    else

                        init_counter = length(init_cond);
                        init_cond{init_counter+1}.continuousSet      = and(X0{m},GUARD);
                        init_cond{init_counter+1}.initialCells            = guards(j);
                        init_cond{init_counter+1}.discreteSet           = disc_state;
                        init_cond{init_counter+1}.initialLocation       = 01;
                        [dum1,dum2,cell_idx]=intersect(guards(j),locations_list{init_loc}.interior_cells);
                        if isempty(cell_idx)
                            locations_list{init_loc}.interior_cells(length(locations_list{init_loc}.interior_cells)+1)= guards(j);
                            
                        end%if
                        
                    end %if
            
                end %if
        
            end %for
    
        end%for
    
    end%for

end%for

initial_conditions      = init_cond;
locations_updated    = locations_list;

return

% -----------------------------------------------------------------------------

function test=is_repeat(boundary,hpflags)

%This function tests if CELLS already contains a region described by the HP's
%in the vector 'boundary'

global CELLS

test=0;
size=length(CELLS);
for i=1:size
   %if isequal(wcommon(CELLS{i}.boundary,boundary),ones(1,length(boundary)))&...
   if length(intersect(boundary,CELLS{i}.boundary))==length(boundary)&...
         length(boundary)==length(CELLS{i}.boundary)
      
     %If all boundary HP's are equal, cell must be tested to see if all HP sides are
     %also equal.  (Added by JPK 6/2002)
     not_eq=0; 
     for j=1:length(boundary)
           [dum1,dum2,member_num]=intersect(boundary(j),CELLS{i}.boundary);
           if hpflags(j)~=CELLS{i}.hpflags(member_num)
                not_eq=1;
                break;
           end
      end
      if ~not_eq
        test=i;
        break;
      end
   end
end

return

%-----------------------------------------------------------------------------

function invariant=intersect_complements(locations,loc)

%This function takes all of the complements of all if the guards for the location 'loc'
%and intersects them.  The resulting regions are the 'invariant'.

global CELLS 

previous_invariant=locations{loc}.transitions{1}.guard_compl;

if length(locations{loc}.transitions)>1
%Intersect the first set of complements with the 2nd.  Then intersect that result
%with the 3rd, and so on...
for j=2:length(locations{loc}.transitions)
   
   new_guard_compl=locations{loc}.transitions{j}.guard_compl;
   invariant=[];
   
   for k=1:length(new_guard_compl)
      
      new_cell=new_guard_compl(k);
      new_hps=CELLS{new_cell}.boundary;
      new_hpflags=CELLS{new_cell}.hpflags;
      new_pthflags=CELLS{new_cell}.pthflags;
      for i=1:length(previous_invariant)
         
         existing_guard_compl=previous_invariant(i);
         old_hps=CELLS{existing_guard_compl}.boundary;
         old_hpflags=CELLS{existing_guard_compl}.hpflags;
         old_pthflags=CELLS{existing_guard_compl}.pthflags;
         [intersection_boundary,intersection_hpflags]=clean_up_boundary(old_hps,new_hps,old_hpflags,new_hpflags);
         if ~isempty(intersection_boundary)
            repeat_test=is_repeat(intersection_boundary,intersection_hpflags);
            if repeat_test~=0
               %************************************************
               %Since this CELL already exists, just point to the existing CELL
               invariant=[invariant repeat_test];
               %Now update existing pthb flag list
               for p=1:length(old_pthflags)
                    if old_pthflags(p)~=-1
                       pthflags(p)=old_pthflags(p);
                    elseif new_pthflags(p)~=-1
                       pthflags(p)=new_pthflags(p);
                    elseif CELLS{repeat_test}.pthflags(p)~=-1
                       pthflags(p)=CELLS{repeat_test}.pthflags(p);
                    else
                       pthflags(p)=-1;
                    end
                 end
					%***********************************************
               
            else
               new_cell=length(CELLS)+1;
               
  					%************************************
  					CELLS{new_cell}.boundary = intersection_boundary;
  					CELLS{new_cell}.hpflags = intersection_hpflags;
                 for p=1:length(old_pthflags)
                    if old_pthflags(p)~=-1
                       pthflags(p)=old_pthflags(p);
                    elseif new_pthflags(p)~=-1
                       pthflags(p)=new_pthflags(p);
                    else
                       pthflags(p)=-1;
                    end
                 end
               CELLS{new_cell}.pthflags = pthflags;
               %*************
               invariant=[invariant new_cell];
            end   
         end
      end
      end
      previous_invariant=invariant;
   
end

else
   invariant=previous_invariant;
end

if isempty(invariant)
   name=locations{loc}.state;
   fprintf(1,'\nWARNING: Location "')
   for i=1:size(name,1)
      fprintf(1,'%s ',name(i,:))
   end
   fprintf(1,'" has no guard invariant.\n')
end

return

% ----------------------------------------------------------------------------

function [boundary_new,hpflags_new] = clean_up_boundary(first_boundary,second_boundary,first_hpflags,second_hpflags)

global GLOBAL_PIHA
global GLOBAL_APPROX_PARAM

hyperplanes = GLOBAL_PIHA.Hyperplanes;

[c1,d1]=cell_ineq(first_boundary,first_hpflags);
[c2,d2]=cell_ineq(second_boundary,second_hpflags);

a=linearcon([],[],c1,d1);
b=linearcon([],[],c2,d2);

if isempty(a&b)
   boundary_new=[];
   hpflags_new=[];
else
boundary=[first_boundary second_boundary];
hpflags=[first_hpflags second_hpflags];

[CBND,dBND] = cell_ineq(boundary,hpflags);
[CE,dE,CI,dI] = linearcon_data(clean_up(linearcon([],[],CBND,dBND)));

%Use the following to indicate which hpflags elements will still be valid
hp_flag_index=[];

if (length(dI) == length(dBND))
  boundary_new = boundary;
else
  hyperplane_tol = GLOBAL_APPROX_PARAM.poly_hyperplane_tol;
  boundary_new = [];
  for k = 1:length(boundary)
    ck = hyperplanes{boundary(k)}.c;
    dk = hyperplanes{boundary(k)}.d;
    found = 0;
    for l = 1:length(dI)
      if rank([CI(l,:) dI(l); ck dk],hyperplane_tol) == 1
        found = 1;
        break;
      end
    end
    if found&~ismember(boundary(k),boundary_new)
      boundary_new = [boundary_new boundary(k)];
      hp_flag_index=[hp_flag_index k];
   end
end

%Create new hpflags vector
for j=1:length(hp_flag_index)
   hpflags_new(j)=hpflags(hp_flag_index(j));
end


end

end

return
%-------------------------------------------------------------------------------------------------------------------------------
%  function clean_transition()
%
% This function eliminates duplicate transitions. It compares the guards of two transitions. 
%If the lists of guards are equal, merge the transitions.
%
function clean_loc=clean_transition(location)
 counter=1;
    for i=1:length(location.transitions)
       if location.transitions{i}.id>0 
         cur_guard=location.transitions{i}.guard;
            for j=i+1:length(location.transitions)
                 if ~isempty(cur_guard)&~isempty(location.transitions{j}.guard)
                         %(isempty(cur_guard)&isempty(location.transitions{j}.guard))
                     if length(cur_guard)==length(location.transitions{j}.guard)
                        if (all(cur_guard==location.transitions{j}.guard))

                        location.transitions{i}.source=[location.transitions{i}.source location.transitions{j}.source];
                        location.transitions{i}.destination=[location.transitions{i}.destination location.transitions{j}.destination];
                        location.transitions{i}.idx=[location.transitions{i}.idx location.transitions{j}.idx];

                        % If either of the transitions resets, we should reset on the composite transion, too. Ansgar
                        location.transitions{i}.reset_flag=max(location.transitions{i}.reset_flag, location.transitions{j}.reset_flag);
                        location.transitions{i}.reset_scs_index=setdiff(...
                            [location.transitions{i}.reset_flag*location.transitions{i}.reset_scs_index, location.transitions{j}.reset_flag*location.transitions{j}.reset_scs_index],0);
                        
                        location.transitions{j}.id=-1;
                        end   %if
                    end %if
                end %if
            end   %for j
     clean_loc.transitions{counter}=location.transitions{i};
     counter=counter+1;
     end   %if location
        
        
    end   %for i
    
    return
